@@ -27,6 +27,14 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: gkclient.h,v $
+ * Revision 1.50  2003/02/12 23:59:22  robertj
+ * Fixed adding missing endpoint identifer in SETUP packet when gatekeeper
+ * routed, pointed out by Stefan Klein
+ * Also fixed correct rutrn of gk routing in IRR packet.
+ *
+ * Revision 1.49  2003/02/07 06:37:42  robertj
+ * Changed registration state to an enum so can determine why the RRQ failed.
+ *
  * Revision 1.48  2003/01/06 07:09:28  robertj
  * Further fixes for alternate gatekeeper, thanks Kevin Bouchard
  *
@@ -324,6 +332,7 @@ class H323Gatekeeper : public H225_RAS
 
       unsigned rejectReason;                      /// Reject reason if returns FALSE
 
+      BOOL gatekeeperRouted;                      /// Flag for call is through gk
       PINDEX endpointCount;                       /// Number of endpoints that can be returned
       H323TransportAddress * transportAddress;    /// Transport address or remote endpoint.
       PBYTEArray * accessTokenData;               /// iNow Gatekeeper Access Token data
@@ -388,7 +397,23 @@ class H323Gatekeeper : public H225_RAS
 
     /**Determine if the endpoint is registered with the gatekeeper.
       */
-    BOOL IsRegistered() const { return isRegistered; }
+    BOOL IsRegistered() const { return registrationFailReason == RegistrationSuccessful; }
+
+    enum RegistrationFailReasons {
+      RegistrationSuccessful,
+      UnregisteredLocally,
+      UnregisteredByGatekeeper,
+      GatekeeperLostRegistration,
+      InvalidListener,
+      DuplicateAlias,
+      SecurityDenied,
+      TransportError,
+      NumRegistrationFailReasons,
+      RegistrationRejectReasonMask = 0x8000
+    };
+    /**Get the registration fail reason.
+     */
+    RegistrationFailReasons GetRegistrationFailReason() const { return registrationFailReason; }
 
     /**Get the gatekeeper name.
        The gets the name of the gatekeeper. It will be of the form id@address
@@ -452,9 +477,9 @@ class H323Gatekeeper : public H225_RAS
 
 
     // Gatekeeper registration state variables
-    BOOL    discoveryComplete;
-    BOOL    isRegistered;
-    PString endpointIdentifier;
+    BOOL     discoveryComplete;
+    PString  endpointIdentifier;
+    RegistrationFailReasons registrationFailReason;
 
     class AlternateInfo : public PObject {
         PCLASSINFO(AlternateInfo, PObject);
