@@ -27,6 +27,30 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: gkserver.h,v $
+ * Revision 1.60  2003/03/04 05:34:13  robertj
+ * Fixed GNU warnings
+ *
+ * Revision 1.59  2003/03/04 05:27:51  robertj
+ * Added statistics for rejected registrations and call admissions.
+ *
+ * Revision 1.58  2003/03/03 06:59:26  robertj
+ * Fixed call SetBandwidth function so adjusts global bandwidth as well.
+ *
+ * Revision 1.57  2003/02/21 07:21:40  robertj
+ * Fixed incorrect ancestor in PCLASSINFO()
+ *
+ * Revision 1.56  2003/02/21 05:28:39  craigs
+ * Factored out code for user with peer elements
+ *
+ * Revision 1.55  2003/02/06 04:43:14  robertj
+ * Changes to help with endpoints behind NAT.
+ *
+ * Revision 1.54  2003/02/03 03:58:06  robertj
+ * Fixed use of local password database with ep's that have multiple aliases.
+ *
+ * Revision 1.53  2003/02/01 13:31:14  robertj
+ * Changes to support CAT authentication in RAS.
+ *
  * Revision 1.52  2002/12/17 01:25:23  robertj
  * Added call backs on gk server when ep indicates alerting or connect.
  *
@@ -231,6 +255,7 @@
 #include "guid.h"
 #include "h235auth.h"
 #include "h323pdu.h"
+#include "h323trans.h"
 
 #include <ptlib/safecoll.h>
 
@@ -287,13 +312,11 @@ class H323GatekeeperRequest : public PObject
 #if PTRACING
     virtual const char * GetName() const = 0;
 #endif
-    virtual const PASN_Sequence & GetRawPDU() const = 0;
     virtual PString GetGatekeeperIdentifier() const = 0;
     virtual unsigned GetGatekeeperRejectTag() const = 0;
     virtual PString GetEndpointIdentifier() const = 0;
     virtual unsigned GetRegisteredEndPointRejectTag() const = 0;
-    virtual const H225_ArrayOf_CryptoH323Token & GetCryptoTokens() const = 0;
-    virtual unsigned GetCryptoTokensField() const = 0;
+    virtual H235Authenticator::ValidationResult ValidatePDU() const = 0;
     virtual unsigned GetSecurityRejectTag() const = 0;
     virtual void SetRejectReason(
       unsigned reasonCode
@@ -301,7 +324,10 @@ class H323GatekeeperRequest : public PObject
 
     BOOL IsFastResponseRequired() const { return fastResponseRequired; }
     H323TransportAddress GetReplyAddress() const { return replyAddresses[0]; }
+    const H323TransportAddressArray & GetReplyAddresses() const { return replyAddresses; }
+    BOOL IsBehindNAT() const { return isBehindNAT; }
     H323GatekeeperListener & GetRasChannel() const { return rasChannel; }
+    H235Authenticator::ValidationResult GetAuthenticatorResult() const { return authenticatorResult; }
 
     PSafePtr<H323RegisteredEndPoint> endpoint;
 
@@ -317,11 +343,8 @@ class H323GatekeeperRequest : public PObject
     H323RasPDU               reject;
     BOOL                     fastResponseRequired;
 
-    enum {
-      UnknownAuthentication,
-      FailedAuthentication,
-      IsAuthenticated
-    } authenticity;
+    H235Authenticator::ValidationResult authenticatorResult;
+    BOOL                                isBehindNAT;
 };
 
 
@@ -337,13 +360,11 @@ class H323GatekeeperGRQ : public H323GatekeeperRequest
 #if PTRACING
     virtual const char * GetName() const;
 #endif
-    virtual const PASN_Sequence & GetRawPDU() const;
     virtual PString GetGatekeeperIdentifier() const;
     virtual unsigned GetGatekeeperRejectTag() const;
     virtual PString GetEndpointIdentifier() const;
     virtual unsigned GetRegisteredEndPointRejectTag() const;
-    virtual const H225_ArrayOf_CryptoH323Token & GetCryptoTokens() const;
-    virtual unsigned GetCryptoTokensField() const;
+    virtual H235Authenticator::ValidationResult ValidatePDU() const;
     virtual unsigned GetSecurityRejectTag() const;
     virtual void SetRejectReason(
       unsigned reasonCode
@@ -370,13 +391,11 @@ class H323GatekeeperRRQ : public H323GatekeeperRequest
 #if PTRACING
     virtual const char * GetName() const;
 #endif
-    virtual const PASN_Sequence & GetRawPDU() const;
     virtual PString GetGatekeeperIdentifier() const;
     virtual unsigned GetGatekeeperRejectTag() const;
     virtual PString GetEndpointIdentifier() const;
     virtual unsigned GetRegisteredEndPointRejectTag() const;
-    virtual const H225_ArrayOf_CryptoH323Token & GetCryptoTokens() const;
-    virtual unsigned GetCryptoTokensField() const;
+    virtual H235Authenticator::ValidationResult ValidatePDU() const;
     virtual unsigned GetSecurityRejectTag() const;
     virtual void SetRejectReason(
       unsigned reasonCode
@@ -403,13 +422,11 @@ class H323GatekeeperURQ : public H323GatekeeperRequest
 #if PTRACING
     virtual const char * GetName() const;
 #endif
-    virtual const PASN_Sequence & GetRawPDU() const;
     virtual PString GetGatekeeperIdentifier() const;
     virtual unsigned GetGatekeeperRejectTag() const;
     virtual PString GetEndpointIdentifier() const;
     virtual unsigned GetRegisteredEndPointRejectTag() const;
-    virtual const H225_ArrayOf_CryptoH323Token & GetCryptoTokens() const;
-    virtual unsigned GetCryptoTokensField() const;
+    virtual H235Authenticator::ValidationResult ValidatePDU() const;
     virtual unsigned GetSecurityRejectTag() const;
     virtual void SetRejectReason(
       unsigned reasonCode
@@ -436,13 +453,11 @@ class H323GatekeeperARQ : public H323GatekeeperRequest
 #if PTRACING
     virtual const char * GetName() const;
 #endif
-    virtual const PASN_Sequence & GetRawPDU() const;
     virtual PString GetGatekeeperIdentifier() const;
     virtual unsigned GetGatekeeperRejectTag() const;
     virtual PString GetEndpointIdentifier() const;
     virtual unsigned GetRegisteredEndPointRejectTag() const;
-    virtual const H225_ArrayOf_CryptoH323Token & GetCryptoTokens() const;
-    virtual unsigned GetCryptoTokensField() const;
+    virtual H235Authenticator::ValidationResult ValidatePDU() const;
     virtual unsigned GetSecurityRejectTag() const;
     virtual void SetRejectReason(
       unsigned reasonCode
@@ -471,13 +486,11 @@ class H323GatekeeperDRQ : public H323GatekeeperRequest
 #if PTRACING
     virtual const char * GetName() const;
 #endif
-    virtual const PASN_Sequence & GetRawPDU() const;
     virtual PString GetGatekeeperIdentifier() const;
     virtual unsigned GetGatekeeperRejectTag() const;
     virtual PString GetEndpointIdentifier() const;
     virtual unsigned GetRegisteredEndPointRejectTag() const;
-    virtual const H225_ArrayOf_CryptoH323Token & GetCryptoTokens() const;
-    virtual unsigned GetCryptoTokensField() const;
+    virtual H235Authenticator::ValidationResult ValidatePDU() const;
     virtual unsigned GetSecurityRejectTag() const;
     virtual void SetRejectReason(
       unsigned reasonCode
@@ -504,13 +517,11 @@ class H323GatekeeperBRQ : public H323GatekeeperRequest
 #if PTRACING
     virtual const char * GetName() const;
 #endif
-    virtual const PASN_Sequence & GetRawPDU() const;
     virtual PString GetGatekeeperIdentifier() const;
     virtual unsigned GetGatekeeperRejectTag() const;
     virtual PString GetEndpointIdentifier() const;
     virtual unsigned GetRegisteredEndPointRejectTag() const;
-    virtual const H225_ArrayOf_CryptoH323Token & GetCryptoTokens() const;
-    virtual unsigned GetCryptoTokensField() const;
+    virtual H235Authenticator::ValidationResult ValidatePDU() const;
     virtual unsigned GetSecurityRejectTag() const;
     virtual void SetRejectReason(
       unsigned reasonCode
@@ -537,13 +548,11 @@ class H323GatekeeperLRQ : public H323GatekeeperRequest
 #if PTRACING
     virtual const char * GetName() const;
 #endif
-    virtual const PASN_Sequence & GetRawPDU() const;
     virtual PString GetGatekeeperIdentifier() const;
     virtual unsigned GetGatekeeperRejectTag() const;
     virtual PString GetEndpointIdentifier() const;
     virtual unsigned GetRegisteredEndPointRejectTag() const;
-    virtual const H225_ArrayOf_CryptoH323Token & GetCryptoTokens() const;
-    virtual unsigned GetCryptoTokensField() const;
+    virtual H235Authenticator::ValidationResult ValidatePDU() const;
     virtual unsigned GetSecurityRejectTag() const;
     virtual void SetRejectReason(
       unsigned reasonCode
@@ -570,13 +579,11 @@ class H323GatekeeperIRR : public H323GatekeeperRequest
 #if PTRACING
     virtual const char * GetName() const;
 #endif
-    virtual const PASN_Sequence & GetRawPDU() const;
     virtual PString GetGatekeeperIdentifier() const;
     virtual unsigned GetGatekeeperRejectTag() const;
     virtual PString GetEndpointIdentifier() const;
     virtual unsigned GetRegisteredEndPointRejectTag() const;
-    virtual const H225_ArrayOf_CryptoH323Token & GetCryptoTokens() const;
-    virtual unsigned GetCryptoTokensField() const;
+    virtual H235Authenticator::ValidationResult ValidatePDU() const;
     virtual unsigned GetSecurityRejectTag() const;
     virtual void SetRejectReason(
       unsigned reasonCode
@@ -757,7 +764,7 @@ class H323GatekeeperCall : public PSafeObject
     const H323TransportAddress & GetDestinationHost() const { return dstHost; }
     PString GetDestinationAddress() const;
     unsigned GetBandwidthUsed() const { return bandwidthUsed; }
-    void SetBandwidthUsed(unsigned bandwidth) { bandwidthUsed = bandwidth; }
+    BOOL SetBandwidthUsed(unsigned bandwidth);
     const PTime & GetLastInfoResponseTime() const { return lastInfoResponse; }
     const PTime & GetCallStartTime() const { return callStartTime; }
     const PTime & GetAlertingTime() const { return alertingTime; }
@@ -974,7 +981,8 @@ class H323RegisteredEndPoint : public PSafeObject
     /**Set password for user activating H.235 security.
       */
     virtual BOOL SetPassword(
-      const PString & password
+      const PString & password,
+      const PString & username = PString::Empty()
     );
 
     /**Get the endpoint identifier assigned to the endpoint.
@@ -1052,6 +1060,10 @@ class H323RegisteredEndPoint : public PSafeObject
       */
     unsigned GetProtocolVersion() const { return protocolVersion; }
 
+    /**Return if gatekeeper thinks the endpoint is behind a firewall.
+      */
+    BOOL IsBehindNAT() const { return isBehindNAT; }
+
     /**Get the flag indicating the endpoint can display credit amounts.
       */
     BOOL CanDisplayAmountString() const { return canDisplayAmountString; }
@@ -1072,6 +1084,7 @@ class H323RegisteredEndPoint : public PSafeObject
     PStringArray              voicePrefixes;
     PCaselessString           applicationInfo;
     unsigned                  protocolVersion;
+    BOOL                      isBehindNAT;
     BOOL                      canDisplayAmountString;
     BOOL                      canEnforceDurationLimit;
     unsigned                  timeToLive;
@@ -1249,9 +1262,9 @@ class H323GatekeeperListener : public H225_RAS
    limited to that. An application would also quite likely descend from this
    class and override call back functions to implement more complex policy.
   */
-class H323GatekeeperServer : public PObject
+class H323GatekeeperServer : public H323TransactionServer
 {
-    PCLASSINFO(H323GatekeeperServer, PObject);
+    PCLASSINFO(H323GatekeeperServer, H323TransactionServer);
   public:
   /**@name Construction */
   //@{
@@ -1266,50 +1279,10 @@ class H323GatekeeperServer : public PObject
     ~H323GatekeeperServer();
   //@}
 
+    WORD GetDefaultUdpPort() { return H225_RAS::DefaultRasUdpPort; }
+
   /**@name Protocol Handler Operations */
   //@{
-    /**Add listeners to the gatekeeper server.
-       If a listener already exists on the interface specified in the list
-       then it is ignored. If a listener does not yet exist a new one is
-       created and if a listener is running that is not in the list then it
-       is stopped and removed.
-
-       If the array is empty then the string "*" is assumed which will listen
-       on the standard UDP port on INADDR_ANY.
-
-       Returns TRUE if at least one interface was successfully started.
-      */
-    BOOL AddListeners(
-      const H323TransportAddressArray & ifaces /// Interfaces to listen on.
-    );
-
-    /**Add a gatekeeper listener to this gatekeeper server given the
-       transport address for the local interface.
-      */
-    BOOL AddListener(
-      const H323TransportAddress & interfaceName
-    );
-
-    /**Add a gatekeeper listener to this gatekeeper server given the transport.
-       Note that the transport is then owned by the listener and will be
-       deleted automatically when the listener is destroyed. Note also the
-       transport is deleted if this function returns FALSE and no listener was
-       created.
-      */
-    BOOL AddListener(
-      H323Transport * transport
-    );
-
-    /**Add a gatekeeper listener to this gatekeeper server.
-       Note that the gatekeeper listener is then owned by the gatekeeper
-       server and will be deleted automatically when the listener is removed.
-       Note also the listener is deleted if this function returns FALSE and
-       the listener was not used.
-      */
-    BOOL AddListener(
-      H323GatekeeperListener * listener
-    );
-
     /**Create a new H323GatkeeperListener.
        The user woiuld not usually use this function as it is used internally
        by the server when new listeners are added by H323TransportAddress.
@@ -1318,16 +1291,10 @@ class H323GatekeeperServer : public PObject
        user defined descendants of H323GatekeeperListener so the user can
        maintain extra information on a interface by interface basis.
       */
-    virtual H323GatekeeperListener * CreateListener(
+    virtual H323Transactor * CreateListener(
       H323Transport * transport  // Transport for listener
     );
 
-    /**Remove a gatekeeper listener from this gatekeeper server.
-       The gatekeeper listener is automatically deleted.
-      */
-    BOOL RemoveListener(
-      H323GatekeeperListener * listener
-    );
   //@}
 
   /**@name EndPoint Operations */
@@ -1665,9 +1632,9 @@ class H323GatekeeperServer : public PObject
     );
 
     /**Get password for user if H.235 security active.
-       Returns FALSE if security active and user not found. Returns TRUE if
-       security active and user is found. Also returns TRUE if security
-       inactive, and the password is always the empty string.
+       Returns TRUE if user is found and password returned. Note the password
+       may be empty in which case the user was found but explicitly ddoes not
+       require security, possibly overriding the requireH235 flag.
       */
     virtual BOOL GetUsersPassword(
       const PString & alias,
@@ -1677,10 +1644,6 @@ class H323GatekeeperServer : public PObject
 
   /**@name Access functions */
   //@{
-    /**Get the owner endpoint.
-     */
-    H323EndPoint & GetOwnerEndPoint() const { return ownerEndPoint; }
-
     /**Get the identifier name for this gatekeeper.
       */
     const PString & GetGatekeeperIdentifier() const { return gatekeeperIdentifier; }
@@ -1747,6 +1710,10 @@ class H323GatekeeperServer : public PObject
       */
     unsigned GetTotalRegistrations() const { return totalRegistrations; }
 
+    /**Get the total registrations rejected since start up.
+      */
+    unsigned GetRejectedRegistrations() const { return rejectedRegistrations; }
+
     /**Get the currently active call count.
       */
     unsigned GetActiveCalls() const { return activeCalls.GetSize(); }
@@ -1758,6 +1725,10 @@ class H323GatekeeperServer : public PObject
     /**Get the total calls since start up.
       */
     unsigned GetTotalCalls() const { return totalCalls; }
+
+    /**Get the total calls rejected since start up.
+      */
+    unsigned GetRejectedCalls() const { return rejectedCalls; }
   //@}
 
   protected:
@@ -1784,7 +1755,6 @@ class H323GatekeeperServer : public PObject
     PStringToString passwords;
 
     // Dynamic variables
-    H323EndPoint & ownerEndPoint;
     PMutex         mutex;
     time_t         identifierBase;
     unsigned       nextIdentifier;
@@ -1811,8 +1781,13 @@ class H323GatekeeperServer : public PObject
 
     PINDEX peakRegistrations;
     PINDEX totalRegistrations;
+    PINDEX rejectedRegistrations;
     PINDEX peakCalls;
     PINDEX totalCalls;
+    PINDEX rejectedCalls;
+
+  friend H323GatekeeperRequest::Response H323GatekeeperRRQ::OnHandlePDU();
+  friend H323GatekeeperRequest::Response H323GatekeeperARQ::OnHandlePDU();
 };
 
 
