@@ -27,6 +27,18 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h225ras.h,v $
+ * Revision 1.33  2003/03/01 00:23:42  craigs
+ * New PeerElement implementation
+ *
+ * Revision 1.32  2003/02/21 07:23:18  robertj
+ * Fixed up some comments
+ *
+ * Revision 1.31  2003/02/21 05:28:39  craigs
+ * Factored out code for user with peer elements
+ *
+ * Revision 1.30  2003/02/01 13:31:14  robertj
+ * Changes to support CAT authentication in RAS.
+ *
  * Revision 1.29  2002/11/28 04:41:44  robertj
  * Added support for RAS ServiceControlIndication command.
  *
@@ -144,6 +156,7 @@
 
 #include "transports.h"
 #include "h235auth.h"
+#include "h323trans.h"
 
 
 class PASN_Sequence;
@@ -499,9 +512,9 @@ class H323CallCreditServiceControl : public H323ServiceControlSession
 
 /**This class embodies the H.225.0 RAS protocol to/from gatekeepers.
   */
-class H225_RAS : public PObject
+class H225_RAS : public H323Transactor
 {
-  PCLASSINFO(H225_RAS, PObject);
+  PCLASSINFO(H225_RAS, H323Transactor);
   public:
   /**@name Construction */
   //@{
@@ -535,7 +548,7 @@ class H225_RAS : public PObject
   //@{
     /**Start the background thread.
       */
-    BOOL StartRasChannel();
+    BOOL StartChannel();
 
     /**Handle and dispatch a RAS PDU..
       */
@@ -704,21 +717,13 @@ class H225_RAS : public PObject
 
     /**Handle unknown PDU type.
       */
-    virtual BOOL OnReceiveUnkown(
+    virtual BOOL OnReceiveUnknown(
       const H323RasPDU & pdu  /// PDU that was not handled.
     );
   //@}
 
   /**@name Member variable access */
   //@{
-    /**Get the gatekeepers associated endpoint.
-      */
-    H323EndPoint & GetEndPoint() const { return endpoint; }
-
-    /**Get the gatekeepers transport channel.
-      */
-    H323Transport & GetTransport() const { return *transport; }
-
     /**Get the gatekeeper identifer.
        For clients at least one successful registration must have been
        achieved for this field to be filling in.
@@ -745,21 +750,17 @@ class H225_RAS : public PObject
       */
     BOOL CheckCryptoTokens(
       const H323RasPDU & pdu,
+      const H225_ArrayOf_ClearToken & clearTokens,
+      unsigned clearOptionalField,
       const H225_ArrayOf_CryptoH323Token & cryptoTokens,
-      unsigned optionalField
+      unsigned cryptoOptionalField
     );
   //@}
 
-
   protected:
     //Background thread handler.
     PDECLARE_NOTIFIER(PThread, H225_RAS, HandleRasChannel);
 	
-    unsigned GetNextSequenceNumber();
-    BOOL SetUpCallSignalAddresses(
-      H225_ArrayOf_TransportAddress & addresses
-    );
-
     class Request : public PObject
     {
         PCLASSINFO(Request, PObject);
@@ -806,19 +807,11 @@ class H225_RAS : public PObject
     void AgeResponses();
     BOOL SendCachedResponse(const H323RasPDU & pdu);
 
-
-    // Configuration variables
-    H323EndPoint  & endpoint;
-    H323Transport * transport;
-
     // Option variables
     PString gatekeeperIdentifier;
     BOOL    checkResponseCryptoTokens;
 
     // Inter-thread synchronisation variables
-    unsigned      nextSequenceNumber;
-    PMutex        nextSequenceNumberMutex;
-
     PDICTIONARY(RequestDict, POrdinalKey, Request);
     RequestDict requests;
     PMutex      requestsMutex;
@@ -840,8 +833,6 @@ class H225_RAS : public PObject
     };
     PSORTED_LIST(ResponseDict, Response);
     ResponseDict responses;
-
-    PMutex pduWriteMutex;
 };
 
 
