@@ -27,6 +27,15 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h323.h,v $
+ * Revision 1.130  2002/11/05 02:18:03  robertj
+ * Changed default for speex codec to be included.
+ *
+ * Revision 1.129  2002/11/04 02:57:24  dereks
+ * Put test around include of speexcodec.h    SPEEX is now an optional codec.
+ *
+ * Revision 1.128  2002/10/24 05:34:03  robertj
+ * Added speex codec
+ *
  * Revision 1.127  2002/10/01 03:07:12  robertj
  * Added version number functions for OpenH323 library itself, plus included
  *   library version in the default vendor information.
@@ -445,6 +454,9 @@
 #include "lpc10codec.h"
 #include "g729codec.h"
 #include "g726codec.h"
+#ifndef NO_SPEEX
+#include "speexcodec.h"
+#endif
 #endif // NO_H323_AUDIO_CODECS
 
 #ifndef NO_H323_VIDEO
