@@ -27,6 +27,11 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h323con.h,v $
+ * Revision 1.59  2003/02/12 23:59:22  robertj
+ * Fixed adding missing endpoint identifer in SETUP packet when gatekeeper
+ * routed, pointed out by Stefan Klein
+ * Also fixed correct rutrn of gk routing in IRR packet.
+ *
  * Revision 1.58  2002/11/27 06:54:52  robertj
  * Added Service Control Session management as per Annex K/H.323 via RAS
  *   only at this stage.
@@ -1965,6 +1970,10 @@ class H323Connection : public PObject
      */
     BOOL HadAnsweredCall() const { return callAnswered; }
 
+    /**Determined if connection is gatekeeper routed.
+     */
+    BOOL IsGatekeeperRouted() const { return gatekeeperRouted; }
+
     /**Get the Q.931 cause code (Q.850) that terminated this call.
        See Q931::CauseValues for common values.
      */
@@ -2184,6 +2193,7 @@ class H323Connection : public PObject
 
     int                  remoteCallWaiting; // Number of call's waiting at the remote endpoint
     BOOL                 callAnswered;
+    BOOL                 gatekeeperRouted;
     unsigned             distinctiveRing;
     PString              callToken;
     unsigned             callReference;
