@@ -27,6 +27,23 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h323ep.h,v $
+ * Revision 1.48  2003/02/13 00:11:31  robertj
+ * Added missing virtual for controlling call transfer, thanks Andrey Pinaev
+ *
+ * Revision 1.47  2003/02/09 00:48:06  robertj
+ * Added function to return if registered with gatekeeper.
+ *
+ * Revision 1.46  2003/02/04 07:06:41  robertj
+ * Added STUN support.
+ *
+ * Revision 1.45  2003/01/26 05:57:58  robertj
+ * Changed ParsePartyName so will accept addresses of the form
+ *   alias@gk:address which will do an LRQ call to "address" using "alias"
+ *   to determine the IP address to connect to.
+ *
+ * Revision 1.44  2003/01/23 02:36:30  robertj
+ * Increased (and made configurable) timeout for H.245 channel TCP connection.
+ *
  * Revision 1.43  2002/11/28 01:19:55  craigs
  * Added virtual to several functions
  *
@@ -204,6 +221,8 @@ class H323ServiceControlSession;
 class OpalT120Protocol;
 class OpalT38Protocol;
 
+class PSTUNClient;
+
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -453,6 +472,10 @@ class H323EndPoint : public PObject
      */
     H323Gatekeeper * GetGatekeeper() const { return gatekeeper; }
 
+    /**Return if endpoint is registered with gatekeeper.
+      */
+    BOOL IsRegisteredWithGatekeeper() const;
+
     /**Unregister and delete the gatekeeper we are registered with.
        The return value indicates FALSE if there was an error during the
        unregistration. However the gatekeeper is still removed and its
@@ -691,11 +714,11 @@ class H323EndPoint : public PObject
        where the default alias is the same as the host, the default transport
        is "ip" and the default port is 1720.
       */
-    void ParsePartyName(
+    BOOL ParsePartyName(
       const PString & party,          /// Party name string.
       PString & alias,                /// Parsed alias name
       H323TransportAddress & address  /// Parsed transport address
-    ) const;
+    );
 
     /**Clear a current connection.
        This hangs up the connection to a remote endpoint. Note that this function
@@ -780,7 +803,7 @@ class H323EndPoint : public PObject
        This gives the application an opportunity to abort the transfer.
        The default behaviour just returns TRUE.
       */
-    BOOL OnCallTransferInitiate(
+    virtual BOOL OnCallTransferInitiate(
       H323Connection & connection,    /// Connection to transfer
       const PString & remoteParty     /// Party transferring to.
     );
@@ -1369,9 +1392,33 @@ class H323EndPoint : public PObject
      */
     void SetInitialBandwidth(unsigned bandwidth) { initialBandwidth = bandwidth; }
 
+    /**Return the STUN server to use.
+       Returns NULL if address is a local address as per IsLocalAddress().
+       Always returns the STUN server if address is zero.
+       Note, the pointer is NOT to be deleted by the user.
+      */
+    PSTUNClient * GetSTUN(
+      const PIPSocket::Address & address = 0
+    ) const;
+
+    /**Set the STUN server address, is of the form host[:port]
+      */
+    void SetSTUNServer(
+      const PString & server
+    );
+
+    /**Determine if the address is "local", ie does not need STUN
+     */
+    virtual BOOL IsLocalAddress(
+      const PIPSocket::Address & remoteAddress
+    ) const;
+
     /**Provide TCP address translation hook
      */
-    virtual void TranslateTCPAddress(PIPSocket::Address & /*localAddr*/, const PIPSocket::Address & /*remoteAddr */) { }
+    virtual void TranslateTCPAddress(
+      PIPSocket::Address & /*localAddr*/,
+      const PIPSocket::Address & /*remoteAddr */
+    ) { }
 
     /**Get the TCP port number base for H.245 channels
      */
@@ -1433,6 +1480,10 @@ class H323EndPoint : public PObject
      */
     const PTimeInterval & GetSignallingChannelCallTimeout() const { return signallingChannelCallTimeout; }
 
+    /**Get the default timeout for incoming H.245 connection.
+     */
+    const PTimeInterval & GetControlChannelStartTimeout() const { return controlChannelStartTimeout; }
+
     /**Get the default timeout for waiting on an end session.
      */
     const PTimeInterval & GetEndSessionTimeout() const { return endSessionTimeout; }
@@ -1609,6 +1660,7 @@ class H323EndPoint : public PObject
     unsigned      soundChannelBuffers;
     BYTE          rtpIpTypeofService;
     PTimeInterval signallingChannelCallTimeout;
+    PTimeInterval controlChannelStartTimeout;
     PTimeInterval endSessionTimeout;
     PTimeInterval masterSlaveDeterminationTimeout;
     unsigned      masterSlaveDeterminationRetries;
@@ -1641,6 +1693,7 @@ class H323EndPoint : public PObject
       WORD   max;
       WORD   current;
     } tcpPorts, udpPorts, rtpIpPorts;
+    PSTUNClient * stun;
 
     /* Protect against absence of a response to the ctIdentify reqest
        (Transferring Endpoint - Call Transfer with a secondary Call) */
