@@ -27,6 +27,18 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h323pdu.h,v $
+ * Revision 1.61  2003/03/01 00:23:42  craigs
+ * New PeerElement implementation
+ *
+ * Revision 1.60  2003/02/25 06:48:15  robertj
+ * More work on PDU transaction abstraction.
+ *
+ * Revision 1.59  2003/02/21 05:28:39  craigs
+ * Factored out code for user with peer elements
+ *
+ * Revision 1.58  2003/02/01 13:31:14  robertj
+ * Changes to support CAT authentication in RAS.
+ *
  * Revision 1.57  2002/11/28 04:41:44  robertj
  * Added support for RAS ServiceControlIndication command.
  *
@@ -231,10 +243,10 @@
 #include <ptlib/sockets.h>
 #include "h323con.h"
 #include "transports.h"
-#include "h235auth.h"
 #include "q931.h"
 #include "h225.h"
 #include "h245.h"
+#include "h323trans.h"
 
 
 class H225_RAS;
@@ -548,7 +560,7 @@ class H323ControlPDU : public H245_MultimediaSystemControlMessage
 
 /**Wrapper class for the H323 gatekeeper RAS channel.
  */
-class H323RasPDU : public H225_RasMessage
+class H323RasPDU : public H225_RasMessage, public H323TransactionPDU
 {
   PCLASSINFO(H323RasPDU, H225_RasMessage);
 
@@ -588,31 +600,16 @@ class H323RasPDU : public H225_RasMessage
     H225_UnknownMessageResponse  & BuildUnknownMessageResponse(unsigned seqNum);
     H225_RequestInProgress       & BuildRequestInProgress(unsigned seqNum, unsigned delay);
 
-    BOOL Read(H323Transport & transport);
-    BOOL Write(H323Transport & transport);
+    const char * GetProtocolName() { return "H225RAS"; }
 
-    unsigned GetSequenceNumber() const;
+    //BOOL Read(H323Transport & transport);
+    //BOOL Write(H323Transport & transport);
 
-    const H235Authenticators & GetAuthenticators() const { return authenticators; }
-    void SetAuthenticators(
-      const H235Authenticators & auth
-    ) { authenticators = auth; }
-
-    BOOL Validate(
-      const H225_ArrayOf_CryptoH323Token & cryptoTokens,
-      const PASN_Sequence & pdu,
-      unsigned optionalField
-    ) const { return authenticators.ValidatePDU(cryptoTokens, pdu, optionalField, rawPDU); }
-
-    void Prepare(
-      H225_ArrayOf_CryptoH323Token & cryptoTokens,
-      PASN_Sequence & pdu,
-      unsigned optionalField
-    ) { authenticators.PreparePDU(cryptoTokens, pdu, optionalField); }
-
-  protected:
-    H235Authenticators authenticators;
-    PPER_Stream        rawPDU;
+    virtual PASN_Object & GetPDU() { return *this; }
+    virtual PASN_Choice & GetChoice() { return *this; }
+    virtual const PASN_Object & GetPDU() const { return *this; }
+    virtual const PASN_Choice & GetChoice() const { return *this; }
+    virtual unsigned GetSequenceNumber() const;
 };
 
 
