@@ -27,6 +27,9 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: lid.h,v $
+ * Revision 1.54  2003/03/05 06:26:40  robertj
+ * Added function to play a WAV file to LID, thanks Pietro Ravasio
+ *
  * Revision 1.53  2002/11/05 04:26:48  robertj
  * Imported RingLine() by array from OPAL.
  *
@@ -833,6 +836,18 @@ class OpalLineInterfaceDevice : public PObject
       unsigned line   /// Number of line
     );
 
+    /**Play a wav file
+      */
+    virtual BOOL PlayAudio(
+      unsigned line,            /// Number of line
+      const PString & filename  /// File Name
+    );
+    
+    /**Stop playing the Wave File
+      */
+    virtual BOOL StopAudio(
+      unsigned line   ///Number of line
+    );
 
 
     /**Dial a number on network line.
