@@ -24,6 +24,10 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: q931.h,v $
+ * Revision 1.51  2003/02/12 00:02:06  robertj
+ * Added more Q.931 cause codes.
+ * Added ability to trace text version of cause codes and IE codes.
+ *
  * Revision 1.50  2002/11/25 22:40:00  robertj
  * Added another Q.850 code
  *
@@ -281,6 +285,8 @@ class Q931 : public PObject
       RedirectingNumberIE  = 0x74,
       UserUserIE           = 0x7e
     };
+    friend ostream & operator<<(ostream & strm, InformationElementCodes ie);
+
     BOOL HasIE(InformationElementCodes ie) const;
     PBYTEArray GetIE(InformationElementCodes ie) const;
     void SetIE(InformationElementCodes ie, const PBYTEArray & userData);
@@ -310,7 +316,7 @@ class Q931 : public PObject
     );
 
     enum CauseValues {
-      UnknownCauseIE,
+      UnknownCauseIE               =  0,
       UnallocatedNumber            =  1,
       NoRouteToNetwork             =  2,
       NoRouteToDestination         =  3,
@@ -338,13 +344,18 @@ class Q931 : public PObject
       Congestion                   = 42,
       RequestedCircuitNotAvailable = 44,
       ResourceUnavailable          = 47,
+      ServiceOptionNotAvailable    = 63,
       InvalidCallReference         = 81,
       ClearedRequestedCallIdentity = 86,
       IncompatibleDestination      = 88,
-      InterworkingUnspecified      = 111,
-      NonStandardReason            = 127,
+      IENonExistantOrNotImplemented= 99,
+      TimerExpiry                  = 102,
+      ProtocolErrorUnspecified     = 111,
+      InterworkingUnspecified      = 127,
       ErrorInCauseIE               = 0x100
     };
+    friend ostream & operator<<(ostream & strm, CauseValues cause);
+
     void SetCause(
       CauseValues value,
       unsigned standard = 0,  // 0 = ITU-T standardized coding
