@@ -27,6 +27,15 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: rtp.h,v $
+ * Revision 1.46  2003/02/07 00:30:41  robertj
+ * Changes for bizarre usage of RTP code outside of scope of H.323 specs.
+ *
+ * Revision 1.45  2003/02/05 06:32:08  robertj
+ * Fixed non-stun symmetric NAT support recently broken.
+ *
+ * Revision 1.44  2003/02/04 07:06:41  robertj
+ * Added STUN support.
+ *
  * Revision 1.43  2002/11/19 01:48:15  robertj
  * Allowed get/set of canonical anme and tool name.
  *
@@ -182,6 +191,7 @@
 
 
 class RTP_JitterBuffer;
+class PSTUNClient;
 
 
 
@@ -959,7 +969,8 @@ class RTP_UDP : public RTP_Session
       PIPSocket::Address localAddress,  /// Local interface to bind to
       WORD portBase,                    /// Base of ports to search
       WORD portMax,                     /// end of ports to search (inclusive)
-      BYTE ipTypeOfService              /// Type of Service byte
+      BYTE ipTypeOfService,             /// Type of Service byte
+      PSTUNClient * stun = NULL         /// STUN server to use createing sockets
     );
   //@}
 
@@ -969,6 +980,12 @@ class RTP_UDP : public RTP_Session
       */
     PIPSocket::Address GetLocalAddress() const { return localAddress; }
 
+    /**Set local address of session.
+      */
+    void SetLocalAddress(
+      const PIPSocket::Address & addr
+    ) { localAddress = addr; }
+
     /**Get remote address of session.
       */
     PIPSocket::Address GetRemoteAddress() const { return remoteAddress; }
@@ -991,11 +1008,11 @@ class RTP_UDP : public RTP_Session
 
     /**Get data UDP socket of session.
       */
-    PUDPSocket & GetDataSocket() { return dataSocket; }
+    PUDPSocket & GetDataSocket() { return *dataSocket; }
 
     /**Get control UDP socket of session.
       */
-    PUDPSocket & GetControlSocket() { return controlSocket; }
+    PUDPSocket & GetControlSocket() { return *controlSocket; }
 
     /**Set the remote address and port information for session.
       */
@@ -1012,7 +1029,7 @@ class RTP_UDP : public RTP_Session
     SendReceiveStatus ReadDataOrControlPDU(
       PUDPSocket & socket,
       PBYTEArray & frame,
-      const char * PTRACE_name
+      BOOL fromDataChannel
     );
 
     PIPSocket::Address localAddress;
@@ -1028,8 +1045,8 @@ class RTP_UDP : public RTP_Session
     BOOL shutdownRead;
     BOOL shutdownWrite;
 
-    PUDPSocket dataSocket;
-    PUDPSocket controlSocket;
+    PUDPSocket * dataSocket;
+    PUDPSocket * controlSocket;
 };
 
 
