@@ -27,6 +27,9 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: transports.h,v $
+ * Revision 1.39  2003/02/06 04:29:23  robertj
+ * Added more support for adding things to H323TransportAddressArrays
+ *
  * Revision 1.38  2002/11/21 06:39:56  robertj
  * Changed promiscuous mode to be three way. Fixes race condition in gkserver
  *   which can cause crashes or more PDUs to be sent to the wrong place.
@@ -267,6 +270,12 @@ class H323TransportAddress : public PString
 
 PDECLARE_ARRAY(H323TransportAddressArray, H323TransportAddress)
   public:
+    H323TransportAddressArray(
+      const H323TransportAddress & address
+    ) { AppendAddress(address); }
+    H323TransportAddressArray(
+      const H225_ArrayOf_TransportAddress & addresses
+    );
     H323TransportAddressArray(
       const PStringArray & array
     ) { AppendStringCollection(array); }
@@ -276,6 +285,17 @@ PDECLARE_ARRAY(H323TransportAddressArray, H323TransportAddress)
     H323TransportAddressArray(
       const PSortedStringList & list
     ) { AppendStringCollection(list); }
+
+    void AppendString(
+      const char * address
+    );
+    void AppendString(
+      const PString & address
+    );
+    void AppendAddress(
+      const H323TransportAddress & address
+    );
+
   protected:
     void AppendStringCollection(
       const PCollection & coll
