@@ -27,6 +27,38 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: gkclient.cxx,v $
+ * Revision 1.138  2003/02/21 05:25:45  craigs
+ * Abstracted out underlying transports for use with peerelements
+ *
+ * Revision 1.137  2003/02/12 23:59:25  robertj
+ * Fixed adding missing endpoint identifer in SETUP packet when gatekeeper
+ * routed, pointed out by Stefan Klein
+ * Also fixed correct rutrn of gk routing in IRR packet.
+ *
+ * Revision 1.136  2003/02/11 04:46:37  robertj
+ * Fixed keep alive RRQ being rejected with full registration required
+ *   reason actually doing a full registration!
+ *
+ * Revision 1.135  2003/02/10 01:51:50  robertj
+ * Fixed bad tokens causing an apparent "transport error", now correctly
+ *   indicates a security error.
+ *
+ * Revision 1.134  2003/02/07 06:38:47  robertj
+ * Changed registration state to an enum so can determine why the RRQ failed.
+ *
+ * Revision 1.133  2003/02/04 07:04:45  robertj
+ * Prevent multiple calls to Connect() if did not change the gk.
+ *
+ * Revision 1.132  2003/02/01 13:31:21  robertj
+ * Changes to support CAT authentication in RAS.
+ *
+ * Revision 1.131  2003/01/11 05:04:03  robertj
+ * Added checks for valid URQ packet, thanks Chih-Wei Huang
+ *
+ * Revision 1.130  2003/01/09 04:45:04  robertj
+ * Fixed problem where if gets GRJ which does not have an alternate gatekeeper
+ *   the system gets into an infinite loop, pointed out by Vladimir Toncar
+ *
  * Revision 1.129  2003/01/06 07:09:43  robertj
  * Further fixes for alternate gatekeeper, thanks Kevin Bouchard
  *
@@ -493,7 +525,7 @@ H323Gatekeeper::H323Gatekeeper(H323EndPoint & ep, H323Transport * trans)
 {
   alternatePermanent = FALSE;
   discoveryComplete = FALSE;
-  isRegistered = FALSE;
+  registrationFailReason = UnregisteredLocally;
 
   pregrantMakeCall = pregrantAnswerCall = RequireARQ;
 
@@ -593,8 +625,16 @@ BOOL H323Gatekeeper::StartDiscovery(const H323TransportAddress & initialAddress)
   discoveryComplete = FALSE;
   unsigned retries = endpoint.GetGatekeeperRequestRetries();
   while (!discoveryComplete) {
-    if (!transport->DiscoverGatekeeper(*this, pdu, address) && --retries == 0)
-      break;
+    if (transport->DiscoverGatekeeper(*this, pdu, address)) {
+      if (address == initialAddress)
+        break;
+      /// If get here must have been GRJ with an alternate gk, start again
+    }
+    else {
+      // Transport failure, retry
+      if (--retries == 0)
+        break;
+    }
   }
 
   requestsMutex.Wait();
@@ -603,7 +643,7 @@ BOOL H323Gatekeeper::StartDiscovery(const H323TransportAddress & initialAddress)
 
   if (discoveryComplete) {
     if (transport->Connect())
-      StartRasChannel();
+      StartChannel();
   }
 
   return discoveryComplete;
@@ -768,7 +808,7 @@ BOOL H323Gatekeeper::RegistrationRequest(BOOL autoReg)
     rrq.m_callCreditCapability.m_canEnforceDurationLimit = TRUE;
   }
 
-  if (isRegistered) {
+  if (IsRegistered()) {
     rrq.IncludeOptionalField(H225_RegistrationRequest::e_keepAlive);
     rrq.m_keepAlive = TRUE;
   }
@@ -781,22 +821,47 @@ BOOL H323Gatekeeper::RegistrationRequest(BOOL autoReg)
     return TRUE;
 
   PTRACE(3, "RAS\tFailed registration of " << endpointIdentifier << " with " << gatekeeperIdentifier);
-  isRegistered = FALSE;
-  switch (request.rejectReason) {
-    case H225_RegistrationRejectReason::e_discoveryRequired :
-      // If have been told by GK that we need to discover it again, set flag
-      // for next register done by timeToLive handler to do discovery
-      requiresDiscovery = TRUE;
-      // Do next case
+  switch (request.responseResult) {
+    case Request::RejectReceived :
+      switch (request.rejectReason) {
+        case H225_RegistrationRejectReason::e_discoveryRequired :
+          // If have been told by GK that we need to discover it again, set flag
+          // for next register done by timeToLive handler to do discovery
+          requiresDiscovery = TRUE;
+          // Do next case
 
-    case H225_RegistrationRejectReason::e_fullRegistrationRequired :
-      // Set timer to retry registration
-      reregisterNow = TRUE;
-      monitorTickle.Signal();
+        case H225_RegistrationRejectReason::e_fullRegistrationRequired :
+          registrationFailReason = GatekeeperLostRegistration;
+          // Set timer to retry registration
+          reregisterNow = TRUE;
+          monitorTickle.Signal();
+          break;
+
+        // Onse below here are permananent errors, so don't try again
+        case H225_RegistrationRejectReason::e_invalidCallSignalAddress :
+          registrationFailReason = InvalidListener;
+          break;
+
+        case H225_RegistrationRejectReason::e_duplicateAlias :
+          registrationFailReason = DuplicateAlias;
+          break;
+
+        case H225_RegistrationRejectReason::e_securityDenial :
+          registrationFailReason = SecurityDenied;
+          break;
+
+        default :
+          registrationFailReason = (RegistrationFailReasons)(request.rejectReason|RegistrationRejectReasonMask);
+          break;
+      }
+      break;
+
+    case Request::BadCryptoTokens :
+      registrationFailReason = SecurityDenied;
       break;
 
     default :
-      // Got a permananent error, so don't try again
+      registrationFailReason = TransportError;
       break;
   }
 
@@ -809,7 +874,7 @@ BOOL H323Gatekeeper::OnReceiveRegistrationConfirm(const H225_RegistrationConfirm
   if (!H225_RAS::OnReceiveRegistrationConfirm(rcf))
     return FALSE;
 
-  isRegistered = TRUE;
+  registrationFailReason = RegistrationSuccessful;
 
   endpointIdentifier = rcf.m_endpointIdentifier;
   PTRACE(3, "RAS\tRegistered " << endpointIdentifier << " with " << gatekeeperIdentifier);
@@ -896,9 +961,6 @@ BOOL H323Gatekeeper::OnReceiveRegistrationReject(const H225_RegistrationReject &
   if (!H225_RAS::OnReceiveRegistrationReject(rrj))
     return FALSE;
 
-  // Registration rejected us, so not registered any more
-  isRegistered = FALSE;
-
   if (rrj.HasOptionalField(H225_RegistrationReject::e_altGKInfo))
     SetAlternates(rrj.m_altGKInfo.m_alternateGatekeeper,
                   rrj.m_altGKInfo.m_altGKisPermanent);
@@ -926,9 +988,8 @@ void H323Gatekeeper::RegistrationTimeToLive()
   }
 
   if (!RegistrationRequest(autoReregister)) {
-
-	PTRACE(2, "RAS\tTime To Live reregistration failed, retrying in 1 minute");
-	timeToLive = PTimeInterval(0, 0, 1);
+    PTRACE_IF(2, !reregisterNow, "RAS\tTime To Live reregistration failed, retrying in 1 minute");
+    timeToLive = PTimeInterval(0, 0, 1);
   }
 }
 
@@ -979,12 +1040,22 @@ BOOL H323Gatekeeper::UnregistrationRequest(int reason)
   if (requestResult)
     return TRUE;
 
-  if (request.responseResult == Request::NoResponseReceived) {
-    isRegistered = FALSE;
-    timeToLive = 0; // zero disables lightweight RRQ
+  switch (request.responseResult) {
+    case Request::NoResponseReceived :
+      registrationFailReason = TransportError;
+      timeToLive = 0; // zero disables lightweight RRQ
+      break;
+
+    case Request::BadCryptoTokens :
+      registrationFailReason = SecurityDenied;
+      timeToLive = 0; // zero disables lightweight RRQ
+      break;
+
+    default :
+      break;
   }
 
-  return !isRegistered;
+  return !IsRegistered();
 }
 
 
@@ -993,7 +1064,7 @@ BOOL H323Gatekeeper::OnReceiveUnregistrationConfirm(const H225_UnregistrationCon
   if (!H225_RAS::OnReceiveUnregistrationConfirm(ucf))
     return FALSE;
 
-  isRegistered = FALSE;
+  registrationFailReason = UnregisteredLocally;
   timeToLive = 0; // zero disables lightweight RRQ
 
   return TRUE;
@@ -1006,10 +1077,22 @@ BOOL H323Gatekeeper::OnReceiveUnregistrationRequest(const H225_UnregistrationReq
     return FALSE;
 
   PTRACE(2, "RAS\tUnregistration received");
+  if (!urq.HasOptionalField(H225_UnregistrationRequest::e_gatekeeperIdentifier) ||
+       urq.m_gatekeeperIdentifier.GetValue() != gatekeeperIdentifier) {
+    PTRACE(1, "RAS\tInconsistent gatekeeperIdentifier!");
+    return FALSE;
+  }
+
+  if (!urq.HasOptionalField(H225_UnregistrationRequest::e_endpointIdentifier) ||
+       urq.m_endpointIdentifier.GetValue() != endpointIdentifier) {
+    PTRACE(1, "RAS\tInconsistent endpointIdentifier!");
+    return FALSE;
+  }
+
   endpoint.ClearAllCalls(H323Connection::EndedByGatekeeper, FALSE);
 
   PTRACE(3, "RAS\tUnregistered, calls cleared");
-  isRegistered = FALSE;
+  registrationFailReason = UnregisteredByGatekeeper;
   timeToLive = 0; // zero disables lightweight RRQ
 
   if (urq.HasOptionalField(H225_UnregistrationRequest::e_alternateGatekeeper))
@@ -1035,7 +1118,7 @@ BOOL H323Gatekeeper::OnReceiveUnregistrationReject(const H225_UnregistrationReje
     return FALSE;
 
   if (lastRequest->rejectReason != H225_UnregRejectReason::e_callInProgress) {
-    isRegistered = FALSE;
+    registrationFailReason = UnregisteredLocally;
     timeToLive = 0; // zero disables lightweight RRQ
   }
 
@@ -1085,6 +1168,7 @@ H323Gatekeeper::AdmissionResponse::AdmissionResponse()
 {
   rejectReason = UINT_MAX;
 
+  gatekeeperRouted = FALSE;
   endpointCount = 1;
   transportAddress = NULL;
   accessTokenData = NULL;
@@ -1095,16 +1179,17 @@ H323Gatekeeper::AdmissionResponse::AdmissionResponse()
 
 
 struct AdmissionRequestResponseInfo {
-  H323Connection * connection;
+  AdmissionRequestResponseInfo(
+    H323Gatekeeper::AdmissionResponse & r,
+    H323Connection & c
+  ) : param(r), connection(c) { }
+
+  H323Gatekeeper::AdmissionResponse & param;
+  H323Connection & connection;
   unsigned allocatedBandwidth;
   unsigned uuiesRequested;
-  PINDEX endpointCount;
-  H323TransportAddress * transportAddress;
-  H225_ArrayOf_AliasAddress * aliasAddresses;
-  H225_ArrayOf_AliasAddress * destExtraCallInfo;
   PString      accessTokenOID1;
   PString      accessTokenOID2;
-  PBYTEArray * accessTokenData;
 };
 
 
@@ -1127,6 +1212,7 @@ BOOL H323Gatekeeper::AdmissionRequest(H323Connection & connection,
         }
         if (response.transportAddress != NULL)
           *response.transportAddress = gkRouteAddress;
+        response.gatekeeperRouted = TRUE;
         return TRUE;
     }
   }
@@ -1188,14 +1274,8 @@ BOOL H323Gatekeeper::AdmissionRequest(H323Connection & connection,
   arq.m_conferenceID = connection.GetConferenceIdentifier();
   arq.m_callIdentifier.m_guid = connection.GetCallIdentifier();
 
-  AdmissionRequestResponseInfo info;
-  info.connection       = &connection;
-  info.endpointCount    = response.endpointCount;
-  info.transportAddress = response.transportAddress;
-  info.aliasAddresses   = response.aliasAddresses;
-  info.destExtraCallInfo= response.destExtraCallInfo;
-  info.accessTokenData  = response.accessTokenData;
-  info.accessTokenOID1  = connection.GetGkAccessTokenOID();
+  AdmissionRequestResponseInfo info(response, connection);
+  info.accessTokenOID1 = connection.GetGkAccessTokenOID();
   PINDEX comma = info.accessTokenOID1.Find(',');
   if (comma == P_MAX_INDEX)
     info.accessTokenOID2 = info.accessTokenOID1;
@@ -1208,31 +1288,47 @@ BOOL H323Gatekeeper::AdmissionRequest(H323Connection & connection,
   request.responseInfo = &info;
 
   if (!authenticators.IsEmpty()) {
-    pdu.Prepare(arq.m_cryptoTokens, arq, H225_AdmissionRequest::e_cryptoTokens);
+    pdu.Prepare(arq.m_tokens, H225_AdmissionRequest::e_tokens,
+                arq.m_cryptoTokens, H225_AdmissionRequest::e_cryptoTokens);
 
     PString remoteId, localId, password;
     if (connection.GetAdmissionRequestAuthentication(arq, remoteId, localId, password)) {
       PTRACE(3, "RAS\tAuthenticators credentials replaced with \"" << localId << "\" during ARQ");
 
       H235Authenticators adjustedAthenticators = authenticators.Adjust(remoteId, localId, password);
-      adjustedAthenticators.PreparePDU(arq.m_cryptoTokens, arq, H225_AdmissionRequest::e_cryptoTokens);
+      adjustedAthenticators.PreparePDU(pdu,
+                                       arq.m_tokens, H225_AdmissionRequest::e_tokens,
+                                       arq.m_cryptoTokens, H225_AdmissionRequest::e_cryptoTokens);
       pdu.SetAuthenticators(adjustedAthenticators);
     }
   }
 
   if (!MakeRequest(request)) {
-    response.rejectReason = request.responseResult == Request::RejectReceived
-                                                ? request.rejectReason : UINT_MAX;
+    response.rejectReason = request.rejectReason;
 
     // See if we are registered.
-    if (response.rejectReason != UINT_MAX &&
+    if (request.responseResult == Request::RejectReceived &&
         response.rejectReason != H225_AdmissionRejectReason::e_callerNotRegistered &&
         response.rejectReason != H225_AdmissionRejectReason::e_invalidEndpointIdentifier)
       return FALSE;
 
     PTRACE(2, "RAS\tEndpoint has become unregistered during ARQ from gatekeeper " << gatekeeperIdentifier);
+
     // Have been told we are not registered (or gk offline)
-    isRegistered = FALSE;
+    switch (request.responseResult) {
+      case Request::NoResponseReceived :
+        registrationFailReason = TransportError;
+        response.rejectReason = UINT_MAX;
+        break;
+
+      case Request::BadCryptoTokens :
+        registrationFailReason = SecurityDenied;
+        response.rejectReason = H225_AdmissionRejectReason::e_securityDenial;
+        break;
+
+      default :
+        registrationFailReason = GatekeeperLostRegistration;
+    }
 
     // If we are not registered and auto register is set ...
     if (!autoReregister)
@@ -1263,8 +1359,6 @@ BOOL H323Gatekeeper::AdmissionRequest(H323Connection & connection,
     }
   }
 
-  response.endpointCount = info.endpointCount;
-
   connection.SetBandwidthAvailable(info.allocatedBandwidth);
   connection.SetUUIEsRequested(info.uuiesRequested);
 
@@ -1343,39 +1437,41 @@ BOOL H323Gatekeeper::OnReceiveAdmissionConfirm(const H225_AdmissionConfirm & acf
 
   AdmissionRequestResponseInfo & info = *(AdmissionRequestResponseInfo *)lastRequest->responseInfo;
   info.allocatedBandwidth = acf.m_bandWidth;
-  if (info.transportAddress != NULL)
-    *info.transportAddress = acf.m_destCallSignalAddress;
+  if (info.param.transportAddress != NULL)
+    *info.param.transportAddress = acf.m_destCallSignalAddress;
+
+  info.param.gatekeeperRouted = acf.m_callModel.GetTag() == H225_CallModel::e_gatekeeperRouted;
 
   // Remove the endpoint aliases that the gatekeeper did not like and add the
   // ones that it really wants us to be.
-  if (info.aliasAddresses != NULL &&
+  if (info.param.aliasAddresses != NULL &&
       acf.HasOptionalField(H225_AdmissionConfirm::e_destinationInfo)) {
     PTRACE(3, "RAS\tGatekeeper specified " << acf.m_destinationInfo.GetSize() << " aliases in ACF");
-    *info.aliasAddresses = acf.m_destinationInfo;
+    *info.param.aliasAddresses = acf.m_destinationInfo;
   }
 
   if (acf.HasOptionalField(H225_AdmissionConfirm::e_uuiesRequested))
     info.uuiesRequested = GetUUIEsRequested(acf.m_uuiesRequested);
 
-  if (info.destExtraCallInfo != NULL &&
+  if (info.param.destExtraCallInfo != NULL &&
       acf.HasOptionalField(H225_AdmissionConfirm::e_destExtraCallInfo))
-    *info.destExtraCallInfo = acf.m_destExtraCallInfo;
+    *info.param.destExtraCallInfo = acf.m_destExtraCallInfo;
 
-  if (info.accessTokenData != NULL && acf.HasOptionalField(H225_AdmissionConfirm::e_tokens))
-    ExtractToken(info, acf.m_tokens, *info.accessTokenData);
+  if (info.param.accessTokenData != NULL && acf.HasOptionalField(H225_AdmissionConfirm::e_tokens))
+    ExtractToken(info, acf.m_tokens, *info.param.accessTokenData);
 
-  if (info.transportAddress != NULL) {
+  if (info.param.transportAddress != NULL) {
     PINDEX count = 1;
-    for (PINDEX i = 0; i < acf.m_alternateEndpoints.GetSize() && count < info.endpointCount; i++) {
+    for (PINDEX i = 0; i < acf.m_alternateEndpoints.GetSize() && count < info.param.endpointCount; i++) {
       if (acf.m_alternateEndpoints[i].HasOptionalField(H225_Endpoint::e_callSignalAddress) &&
           acf.m_alternateEndpoints[i].m_callSignalAddress.GetSize() > 0) {
-        info.transportAddress[count] = acf.m_alternateEndpoints[i].m_callSignalAddress[0];
-        if (info.accessTokenData != NULL)
-          ExtractToken(info, acf.m_alternateEndpoints[i].m_tokens, info.accessTokenData[count]);
+        info.param.transportAddress[count] = acf.m_alternateEndpoints[i].m_callSignalAddress[0];
+        if (info.param.accessTokenData != NULL)
+          ExtractToken(info, acf.m_alternateEndpoints[i].m_tokens, info.param.accessTokenData[count]);
         count++;
       }
     }
-    info.endpointCount = count;
+    info.param.endpointCount = count;
   }
 
   if (acf.HasOptionalField(H225_AdmissionConfirm::e_irrFrequency))
@@ -1383,7 +1479,7 @@ BOOL H323Gatekeeper::OnReceiveAdmissionConfirm(const H225_AdmissionConfirm & acf
   willRespondToIRR = acf.m_willRespondToIRR;
 
   if (acf.HasOptionalField(H225_AdmissionConfirm::e_serviceControl))
-    OnServiceControlSessions(acf.m_serviceControl, info.connection);
+    OnServiceControlSessions(acf.m_serviceControl, &info.connection);
 
   return TRUE;
 }
@@ -1396,7 +1492,7 @@ BOOL H323Gatekeeper::OnReceiveAdmissionReject(const H225_AdmissionReject & arj)
 
   if (arj.HasOptionalField(H225_AdmissionConfirm::e_serviceControl))
     OnServiceControlSessions(arj.m_serviceControl,
-              ((AdmissionRequestResponseInfo *)lastRequest->responseInfo)->connection);
+              &((AdmissionRequestResponseInfo *)lastRequest->responseInfo)->connection);
 
   return TRUE;
 }
@@ -1651,7 +1747,8 @@ static void AddInfoRequestResponseCall(H225_InfoRequestResponse & irr,
 
   info.m_callType.SetTag(H225_CallType::e_pointToPoint);
   info.m_bandWidth = connection.GetBandwidthUsed();
-  info.m_callModel.SetTag(H225_CallModel::e_direct);
+  info.m_callModel.SetTag(connection.IsGatekeeperRouted() ? H225_CallModel::e_gatekeeperRouted
+                                                          : H225_CallModel::e_direct);
 
   info.IncludeOptionalField(H225_InfoRequestResponse_perCallInfo_subtype::e_usageInformation);
   SetRasUsageInformation(connection, info.m_usageInformation);
@@ -1917,7 +2014,18 @@ BOOL H323Gatekeeper::MakeRequestWithReregister(Request & request, unsigned unreg
   PTRACE(2, "RAS\tEndpoint has become unregistered from gatekeeper " << gatekeeperIdentifier);
 
   // Have been told we are not registered (or gk offline)
-  isRegistered = FALSE;
+  switch (request.responseResult) {
+    case Request::NoResponseReceived :
+      registrationFailReason = TransportError;
+      break;
+
+    case Request::BadCryptoTokens :
+      registrationFailReason = SecurityDenied;
+      break;
+
+    default :
+      registrationFailReason = GatekeeperLostRegistration;
+  }
 
   // If we are not registered and auto register is set ...
   if (!autoReregister)
@@ -1950,13 +2058,15 @@ BOOL H323Gatekeeper::MakeRequest(Request & request)
      transport address of the alternate while the other is in timeout. We
      have to block the function */
 
-  H323TransportAddress tempAddr = transport->GetRemoteAddress ();
+  H323TransportAddress tempAddr = transport->GetRemoteAddress();
   PString tempIdentifier = gatekeeperIdentifier;
   
   PINDEX alt = 0;
   for (;;) {
     if (H225_RAS::MakeRequest(request)) {
-      if (!alternatePermanent) 
+      if (!alternatePermanent &&
+            (transport->GetRemoteAddress() != tempAddr ||
+             gatekeeperIdentifier != tempIdentifier))
         Connect(tempAddr, tempIdentifier);
       requestMutex.Signal();
       return TRUE;
@@ -1989,12 +2099,12 @@ BOOL H323Gatekeeper::MakeRequest(Request & request)
       transport->SetRemoteAddress (altInfo->rasAddress);
       transport->Connect();
       gatekeeperIdentifier = altInfo->gatekeeperIdentifier;
-      StartRasChannel();
+      StartChannel();
     } while (altInfo->registrationState == AlternateInfo::RegistrationFailed);
     
     if (altInfo->registrationState == AlternateInfo::NeedToRegister) {
       altInfo->registrationState = AlternateInfo::RegistrationFailed;
-      isRegistered = FALSE;
+      registrationFailReason = TransportError;
       discoveryComplete = FALSE;
       H323RasPDU pdu;
       Request req(SetupGatekeeperRequest(pdu), pdu);
