@@ -27,6 +27,18 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h225ras.cxx,v $
+ * Revision 1.49  2003/03/01 00:21:01  craigs
+ * Fixed spelling mistake!
+ *
+ * Revision 1.48  2003/02/21 05:25:45  craigs
+ * Abstracted out underlying transports for use with peerelements
+ *
+ * Revision 1.47  2003/02/10 04:18:55  robertj
+ * Made RAS read loop more robust.
+ *
+ * Revision 1.46  2003/02/01 13:31:21  robertj
+ * Changes to support CAT authentication in RAS.
+ *
  * Revision 1.45  2002/12/18 06:24:52  robertj
  * Improved logging around the RAS poll function, on timeouts etc
  *
@@ -427,27 +439,17 @@ void H323CallCreditServiceControl::OnChange(unsigned /*type*/,
 /////////////////////////////////////////////////////////////////////////////
 
 H225_RAS::H225_RAS(H323EndPoint & ep, H323Transport * trans)
-  : endpoint(ep)
+  : H323Transactor(ep, trans, DefaultRasUdpPort)
 {
-  if (trans != NULL)
-    transport = trans;
-  else
-    transport = new H323TransportUDP(ep, INADDR_ANY, DefaultRasUdpPort);
-
   checkResponseCryptoTokens = TRUE;
 
   lastRequest = NULL;
-  nextSequenceNumber = PRandom::Number()%65536;
   requests.DisallowDeleteObjects();
 }
 
 
 H225_RAS::~H225_RAS()
 {
-  if (transport != NULL) {
-    transport->CleanUpOnTermination();
-    delete transport;
-  }
 }
 
 
@@ -462,7 +464,7 @@ void H225_RAS::PrintOn(ostream & strm) const
 }
 
 
-BOOL H225_RAS::StartRasChannel()
+BOOL H225_RAS::StartChannel()
 {
   transport->AttachThread(PThread::Create(PCREATE_NOTIFIER(HandleRasChannel), 0,
                                           PThread::NoAutoDeleteThread,
@@ -474,12 +476,17 @@ BOOL H225_RAS::StartRasChannel()
 
 void H225_RAS::HandleRasChannel(PThread &, INT)
 {
+  PTRACE(2, "RAS\tStarting RAS listener thread on " << *transport);
+
   transport->SetReadTimeout(PMaxTimeInterval);
   H323RasPDU response;
 
+  PINDEX consecutiveErrors = 0;
+
   for (;;) {
     PTRACE(5, "RAS\tReading PDU");
     if (response.Read(*transport)) {
+      consecutiveErrors = 0;
       lastRequest = NULL;
       if (HandleRasPDU(response))
         lastRequest->responseHandled.Signal();
@@ -487,15 +494,30 @@ void H225_RAS::HandleRasChannel(PThread &, INT)
         lastRequest->responseMutex.Signal();
     }
     else {
-      switch (transport->GetErrorNumber()) {
-        case ECONNRESET:
-        case ECONNREFUSED:
-          PTRACE(2, "RAS\tCannot access remote " << transport->GetRemoteAddress());
-          break;
+      switch (transport->GetErrorCode(PChannel::LastReadError)) {
+        case PChannel::Interrupted :
+          if (transport->IsOpen())
+            break;
+          // Do NotOpen case
 
-        default:
-          PTRACE(1, "RAS\tRead error: " << transport->GetErrorText(PChannel::LastReadError));
+        case PChannel::NotOpen :
+          PTRACE(2, "RAS\tRead socket closed, aborting RAS listener thread on " << *transport);
           return;
+
+        default :
+          switch (transport->GetErrorNumber(PChannel::LastReadError)) {
+            case ECONNRESET:
+            case ECONNREFUSED:
+              PTRACE(2, "RAS\tCannot access remote " << transport->GetRemoteAddress());
+              break;
+
+            default:
+              PTRACE(1, "RAS\tRead error: " << transport->GetErrorText(PChannel::LastReadError));
+              if (++consecutiveErrors > 10) {
+                PTRACE(2, "RAS\tAborting RAS listener thread on " << *transport);
+                return;
+              }
+          }
       }
     }
   }
@@ -636,7 +658,7 @@ BOOL H225_RAS::HandleRasPDU(const H323RasPDU & pdu)
       return OnReceiveServiceControlResponse(pdu, pdu);
 
     default :
-      OnReceiveUnkown(pdu);
+      OnReceiveUnknown(pdu);
   }
 
   return FALSE;
@@ -866,19 +888,6 @@ BOOL H225_RAS::CheckForResponse(unsigned reqTag, unsigned seqNum, const PASN_Cho
 }
 
 
-BOOL H225_RAS::SetUpCallSignalAddresses(H225_ArrayOf_TransportAddress & addresses)
-{
-  H225_TransportAddress rasAddress;
-  transport->SetUpTransportPDU(rasAddress, TRUE);
-
-  const H323ListenerList & listeners = endpoint.GetListeners();
-  for (PINDEX i = 0; i < listeners.GetSize(); i++)
-    listeners[i].SetUpTransportPDU(addresses, *transport);
-
-  return addresses.GetSize() > 0;
-}
-
-
 BOOL H225_RAS::OnReceiveRequestInProgress(const H323RasPDU &, const H225_RequestInProgress & rip)
 {
   unsigned seqNum = rip.m_requestSeqNum;
@@ -1012,7 +1021,8 @@ BOOL H225_RAS::OnReceiveGatekeeperReject(const H225_GatekeeperReject & /*grj*/)
 void H225_RAS::OnSendRegistrationRequest(H323RasPDU & pdu, H225_RegistrationRequest & rrq)
 {
   OnSendRegistrationRequest(rrq);
-  pdu.Prepare(rrq.m_cryptoTokens, rrq, H225_RegistrationRequest::e_cryptoTokens);
+  pdu.Prepare(rrq.m_tokens, H225_RegistrationRequest::e_tokens,
+              rrq.m_cryptoTokens, H225_RegistrationRequest::e_cryptoTokens);
 }
 
 
@@ -1030,7 +1040,8 @@ void H225_RAS::OnSendRegistrationConfirm(H323RasPDU & pdu, H225_RegistrationConf
 
   OnSendRegistrationConfirm(rcf);
 
-  pdu.Prepare(rcf.m_cryptoTokens, rcf, H225_RegistrationConfirm::e_cryptoTokens);
+  pdu.Prepare(rcf.m_tokens, H225_RegistrationConfirm::e_tokens,
+              rcf.m_cryptoTokens, H225_RegistrationConfirm::e_cryptoTokens);
 }
 
 
@@ -1048,7 +1059,8 @@ void H225_RAS::OnSendRegistrationReject(H323RasPDU & pdu, H225_RegistrationRejec
 
   OnSendRegistrationReject(rrj);
 
-  pdu.Prepare(rrj.m_cryptoTokens, rrj, H225_RegistrationReject::e_cryptoTokens);
+  pdu.Prepare(rrj.m_tokens, H225_RegistrationReject::e_tokens,
+              rrj.m_cryptoTokens, H225_RegistrationReject::e_cryptoTokens);
 }
 
 
@@ -1084,7 +1096,9 @@ BOOL H225_RAS::OnReceiveRegistrationConfirm(const H323RasPDU & pdu, const H225_R
     }
   }
 
-  if (!CheckCryptoTokens(pdu, rcf.m_cryptoTokens, H225_RegistrationConfirm::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         rcf.m_tokens, H225_RegistrationConfirm::e_tokens,
+                         rcf.m_cryptoTokens, H225_RegistrationConfirm::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveRegistrationConfirm(rcf);
@@ -1102,7 +1116,9 @@ BOOL H225_RAS::OnReceiveRegistrationReject(const H323RasPDU & pdu, const H225_Re
   if (!CheckForResponse(H225_RasMessage::e_registrationRequest, rrj.m_requestSeqNum, &rrj.m_rejectReason))
     return FALSE;
 
-  if (!CheckCryptoTokens(pdu, rrj.m_cryptoTokens, H225_RegistrationReject::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         rrj.m_tokens, H225_RegistrationReject::e_tokens,
+                         rrj.m_cryptoTokens, H225_RegistrationReject::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveRegistrationReject(rrj);
@@ -1118,7 +1134,8 @@ BOOL H225_RAS::OnReceiveRegistrationReject(const H225_RegistrationReject & /*rrj
 void H225_RAS::OnSendUnregistrationRequest(H323RasPDU & pdu, H225_UnregistrationRequest & urq)
 {
   OnSendUnregistrationRequest(urq);
-  pdu.Prepare(urq.m_cryptoTokens, urq, H225_UnregistrationRequest::e_cryptoTokens);
+  pdu.Prepare(urq.m_tokens, H225_UnregistrationRequest::e_tokens,
+              urq.m_cryptoTokens, H225_UnregistrationRequest::e_cryptoTokens);
 }
 
 
@@ -1130,7 +1147,8 @@ void H225_RAS::OnSendUnregistrationRequest(H225_UnregistrationRequest & /*urq*/)
 void H225_RAS::OnSendUnregistrationConfirm(H323RasPDU & pdu, H225_UnregistrationConfirm & ucf)
 {
   OnSendUnregistrationConfirm(ucf);
-  pdu.Prepare(ucf.m_cryptoTokens, ucf, H225_UnregistrationConfirm::e_cryptoTokens);
+  pdu.Prepare(ucf.m_tokens, H225_UnregistrationConfirm::e_tokens,
+              ucf.m_cryptoTokens, H225_UnregistrationConfirm::e_cryptoTokens);
 }
 
 
@@ -1142,7 +1160,8 @@ void H225_RAS::OnSendUnregistrationConfirm(H225_UnregistrationConfirm & /*ucf*/)
 void H225_RAS::OnSendUnregistrationReject(H323RasPDU & pdu, H225_UnregistrationReject & urj)
 {
   OnSendUnregistrationReject(urj);
-  pdu.Prepare(urj.m_cryptoTokens, urj, H225_UnregistrationReject::e_cryptoTokens);
+  pdu.Prepare(urj.m_tokens, H225_UnregistrationReject::e_tokens,
+              urj.m_cryptoTokens, H225_UnregistrationReject::e_cryptoTokens);
 }
 
 
@@ -1153,7 +1172,9 @@ void H225_RAS::OnSendUnregistrationReject(H225_UnregistrationReject & /*urj*/)
 
 BOOL H225_RAS::OnReceiveUnregistrationRequest(const H323RasPDU & pdu, const H225_UnregistrationRequest & urq)
 {
-  if (!CheckCryptoTokens(pdu, urq.m_cryptoTokens, H225_UnregistrationRequest::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         urq.m_tokens, H225_UnregistrationRequest::e_tokens,
+                         urq.m_cryptoTokens, H225_UnregistrationRequest::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveUnregistrationRequest(urq);
@@ -1171,7 +1192,9 @@ BOOL H225_RAS::OnReceiveUnregistrationConfirm(const H323RasPDU & pdu, const H225
   if (!CheckForResponse(H225_RasMessage::e_unregistrationRequest, ucf.m_requestSeqNum))
     return FALSE;
 
-  if (!CheckCryptoTokens(pdu, ucf.m_cryptoTokens, H225_UnregistrationConfirm::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         ucf.m_tokens, H225_UnregistrationConfirm::e_tokens,
+                         ucf.m_cryptoTokens, H225_UnregistrationConfirm::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveUnregistrationConfirm(ucf);
@@ -1189,7 +1212,9 @@ BOOL H225_RAS::OnReceiveUnregistrationReject(const H323RasPDU & pdu, const H225_
   if (!CheckForResponse(H225_RasMessage::e_unregistrationRequest, urj.m_requestSeqNum, &urj.m_rejectReason))
     return FALSE;
 
-  if (!CheckCryptoTokens(pdu, urj.m_cryptoTokens, H225_UnregistrationReject::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         urj.m_tokens, H225_UnregistrationReject::e_tokens,
+                         urj.m_cryptoTokens, H225_UnregistrationReject::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveUnregistrationReject(urj);
@@ -1205,7 +1230,8 @@ BOOL H225_RAS::OnReceiveUnregistrationReject(const H225_UnregistrationReject & /
 void H225_RAS::OnSendAdmissionRequest(H323RasPDU & pdu, H225_AdmissionRequest & arq)
 {
   OnSendAdmissionRequest(arq);
-  pdu.Prepare(arq.m_cryptoTokens, arq, H225_AdmissionRequest::e_cryptoTokens);
+  pdu.Prepare(arq.m_tokens, H225_AdmissionRequest::e_tokens,
+              arq.m_cryptoTokens, H225_AdmissionRequest::e_cryptoTokens);
 }
 
 
@@ -1217,7 +1243,8 @@ void H225_RAS::OnSendAdmissionRequest(H225_AdmissionRequest & /*arq*/)
 void H225_RAS::OnSendAdmissionConfirm(H323RasPDU & pdu, H225_AdmissionConfirm & acf)
 {
   OnSendAdmissionConfirm(acf);
-  pdu.Prepare(acf.m_cryptoTokens, acf, H225_AdmissionConfirm::e_cryptoTokens);
+  pdu.Prepare(acf.m_tokens, H225_AdmissionConfirm::e_tokens,
+              acf.m_cryptoTokens, H225_AdmissionConfirm::e_cryptoTokens);
 }
 
 
@@ -1229,7 +1256,8 @@ void H225_RAS::OnSendAdmissionConfirm(H225_AdmissionConfirm & /*acf*/)
 void H225_RAS::OnSendAdmissionReject(H323RasPDU & pdu, H225_AdmissionReject & arj)
 {
   OnSendAdmissionReject(arj);
-  pdu.Prepare(arj.m_cryptoTokens, arj, H225_AdmissionReject::e_cryptoTokens);
+  pdu.Prepare(arj.m_tokens, H225_AdmissionReject::e_tokens,
+              arj.m_cryptoTokens, H225_AdmissionReject::e_cryptoTokens);
 }
 
 
@@ -1240,7 +1268,9 @@ void H225_RAS::OnSendAdmissionReject(H225_AdmissionReject & /*arj*/)
 
 BOOL H225_RAS::OnReceiveAdmissionRequest(const H323RasPDU & pdu, const H225_AdmissionRequest & arq)
 {
-  if (!CheckCryptoTokens(pdu, arq.m_cryptoTokens, H225_AdmissionRequest::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         arq.m_tokens, H225_AdmissionRequest::e_tokens,
+                         arq.m_cryptoTokens, H225_AdmissionRequest::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveAdmissionRequest(arq);
@@ -1258,7 +1288,9 @@ BOOL H225_RAS::OnReceiveAdmissionConfirm(const H323RasPDU & pdu, const H225_Admi
   if (!CheckForResponse(H225_RasMessage::e_admissionRequest, acf.m_requestSeqNum))
     return FALSE;
 
-  if (!CheckCryptoTokens(pdu, acf.m_cryptoTokens, H225_AdmissionConfirm::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         acf.m_tokens, H225_AdmissionConfirm::e_tokens,
+                         acf.m_cryptoTokens, H225_AdmissionConfirm::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveAdmissionConfirm(acf);
@@ -1276,7 +1308,9 @@ BOOL H225_RAS::OnReceiveAdmissionReject(const H323RasPDU & pdu, const H225_Admis
   if (!CheckForResponse(H225_RasMessage::e_admissionRequest, arj.m_requestSeqNum, &arj.m_rejectReason))
     return FALSE;
 
-  if (!CheckCryptoTokens(pdu, arj.m_cryptoTokens, H225_AdmissionReject::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         arj.m_tokens, H225_AdmissionReject::e_tokens,
+                         arj.m_cryptoTokens, H225_AdmissionReject::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveAdmissionReject(arj);
@@ -1292,7 +1326,8 @@ BOOL H225_RAS::OnReceiveAdmissionReject(const H225_AdmissionReject & /*arj*/)
 void H225_RAS::OnSendBandwidthRequest(H323RasPDU & pdu, H225_BandwidthRequest & brq)
 {
   OnSendBandwidthRequest(brq);
-  pdu.Prepare(brq.m_cryptoTokens, brq, H225_BandwidthRequest::e_cryptoTokens);
+  pdu.Prepare(brq.m_tokens, H225_BandwidthRequest::e_tokens,
+              brq.m_cryptoTokens, H225_BandwidthRequest::e_cryptoTokens);
 }
 
 
@@ -1303,7 +1338,9 @@ void H225_RAS::OnSendBandwidthRequest(H225_BandwidthRequest & /*brq*/)
 
 BOOL H225_RAS::OnReceiveBandwidthRequest(const H323RasPDU & pdu, const H225_BandwidthRequest & brq)
 {
-  if (!CheckCryptoTokens(pdu, brq.m_cryptoTokens, H225_BandwidthRequest::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         brq.m_tokens, H225_BandwidthRequest::e_tokens,
+                         brq.m_cryptoTokens, H225_BandwidthRequest::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveBandwidthRequest(brq);
@@ -1319,7 +1356,8 @@ BOOL H225_RAS::OnReceiveBandwidthRequest(const H225_BandwidthRequest & /*brq*/)
 void H225_RAS::OnSendBandwidthConfirm(H323RasPDU & pdu, H225_BandwidthConfirm & bcf)
 {
   OnSendBandwidthConfirm(bcf);
-  pdu.Prepare(bcf.m_cryptoTokens, bcf, H225_BandwidthConfirm::e_cryptoTokens);
+  pdu.Prepare(bcf.m_tokens, H225_BandwidthConfirm::e_tokens,
+              bcf.m_cryptoTokens, H225_BandwidthConfirm::e_cryptoTokens);
 }
 
 
@@ -1333,7 +1371,9 @@ BOOL H225_RAS::OnReceiveBandwidthConfirm(const H323RasPDU & pdu, const H225_Band
   if (!CheckForResponse(H225_RasMessage::e_bandwidthRequest, bcf.m_requestSeqNum))
     return FALSE;
 
-  if (!CheckCryptoTokens(pdu, bcf.m_cryptoTokens, H225_BandwidthConfirm::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         bcf.m_tokens, H225_BandwidthConfirm::e_tokens,
+                         bcf.m_cryptoTokens, H225_BandwidthConfirm::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveBandwidthConfirm(bcf);
@@ -1349,7 +1389,8 @@ BOOL H225_RAS::OnReceiveBandwidthConfirm(const H225_BandwidthConfirm & /*bcf*/)
 void H225_RAS::OnSendBandwidthReject(H323RasPDU & pdu, H225_BandwidthReject & brj)
 {
   OnSendBandwidthReject(brj);
-  pdu.Prepare(brj.m_cryptoTokens, brj, H225_BandwidthReject::e_cryptoTokens);
+  pdu.Prepare(brj.m_tokens, H225_BandwidthReject::e_tokens,
+              brj.m_cryptoTokens, H225_BandwidthReject::e_cryptoTokens);
 }
 
 
@@ -1363,7 +1404,9 @@ BOOL H225_RAS::OnReceiveBandwidthReject(const H323RasPDU & pdu, const H225_Bandw
   if (!CheckForResponse(H225_RasMessage::e_bandwidthRequest, brj.m_requestSeqNum, &brj.m_rejectReason))
     return FALSE;
 
-  if (!CheckCryptoTokens(pdu, brj.m_cryptoTokens, H225_BandwidthReject::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         brj.m_tokens, H225_BandwidthReject::e_tokens,
+                         brj.m_cryptoTokens, H225_BandwidthReject::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveBandwidthReject(brj);
@@ -1379,7 +1422,8 @@ BOOL H225_RAS::OnReceiveBandwidthReject(const H225_BandwidthReject & /*brj*/)
 void H225_RAS::OnSendDisengageRequest(H323RasPDU & pdu, H225_DisengageRequest & drq)
 {
   OnSendDisengageRequest(drq);
-  pdu.Prepare(drq.m_cryptoTokens, drq, H225_DisengageRequest::e_cryptoTokens);
+  pdu.Prepare(drq.m_tokens, H225_DisengageRequest::e_tokens,
+              drq.m_cryptoTokens, H225_DisengageRequest::e_cryptoTokens);
 }
 
 
@@ -1390,7 +1434,9 @@ void H225_RAS::OnSendDisengageRequest(H225_DisengageRequest & /*drq*/)
 
 BOOL H225_RAS::OnReceiveDisengageRequest(const H323RasPDU & pdu, const H225_DisengageRequest & drq)
 {
-  if (!CheckCryptoTokens(pdu, drq.m_cryptoTokens, H225_DisengageRequest::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         drq.m_tokens, H225_DisengageRequest::e_tokens,
+                         drq.m_cryptoTokens, H225_DisengageRequest::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveDisengageRequest(drq);
@@ -1406,7 +1452,8 @@ BOOL H225_RAS::OnReceiveDisengageRequest(const H225_DisengageRequest & /*drq*/)
 void H225_RAS::OnSendDisengageConfirm(H323RasPDU & pdu, H225_DisengageConfirm & dcf)
 {
   OnSendDisengageConfirm(dcf);
-  pdu.Prepare(dcf.m_cryptoTokens, dcf, H225_DisengageConfirm::e_cryptoTokens);
+  pdu.Prepare(dcf.m_tokens, H225_DisengageConfirm::e_tokens,
+              dcf.m_cryptoTokens, H225_DisengageConfirm::e_cryptoTokens);
 }
 
 
@@ -1420,7 +1467,9 @@ BOOL H225_RAS::OnReceiveDisengageConfirm(const H323RasPDU & pdu, const H225_Dise
   if (!CheckForResponse(H225_RasMessage::e_disengageRequest, dcf.m_requestSeqNum))
     return FALSE;
 
-  if (!CheckCryptoTokens(pdu, dcf.m_cryptoTokens, H225_DisengageConfirm::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         dcf.m_tokens, H225_DisengageConfirm::e_tokens,
+                         dcf.m_cryptoTokens, H225_DisengageConfirm::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveDisengageConfirm(dcf);
@@ -1436,7 +1485,8 @@ BOOL H225_RAS::OnReceiveDisengageConfirm(const H225_DisengageConfirm & /*dcf*/)
 void H225_RAS::OnSendDisengageReject(H323RasPDU & pdu, H225_DisengageReject & drj)
 {
   OnSendDisengageReject(drj);
-  pdu.Prepare(drj.m_cryptoTokens, drj, H225_DisengageReject::e_cryptoTokens);
+  pdu.Prepare(drj.m_tokens, H225_DisengageReject::e_tokens,
+              drj.m_cryptoTokens, H225_DisengageReject::e_cryptoTokens);
 }
 
 
@@ -1450,7 +1500,9 @@ BOOL H225_RAS::OnReceiveDisengageReject(const H323RasPDU & pdu, const H225_Disen
   if (!CheckForResponse(H225_RasMessage::e_disengageRequest, drj.m_requestSeqNum, &drj.m_rejectReason))
     return FALSE;
 
-  if (!CheckCryptoTokens(pdu, drj.m_cryptoTokens, H225_DisengageReject::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         drj.m_tokens, H225_DisengageReject::e_tokens,
+                         drj.m_cryptoTokens, H225_DisengageReject::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveDisengageReject(drj);
@@ -1466,7 +1518,8 @@ BOOL H225_RAS::OnReceiveDisengageReject(const H225_DisengageReject & /*drj*/)
 void H225_RAS::OnSendLocationRequest(H323RasPDU & pdu, H225_LocationRequest & lrq)
 {
   OnSendLocationRequest(lrq);
-  pdu.Prepare(lrq.m_cryptoTokens, lrq, H225_LocationRequest::e_cryptoTokens);
+  pdu.Prepare(lrq.m_tokens, H225_LocationRequest::e_tokens,
+              lrq.m_cryptoTokens, H225_LocationRequest::e_cryptoTokens);
 }
 
 
@@ -1477,7 +1530,9 @@ void H225_RAS::OnSendLocationRequest(H225_LocationRequest & /*lrq*/)
 
 BOOL H225_RAS::OnReceiveLocationRequest(const H323RasPDU & pdu, const H225_LocationRequest & lrq)
 {
-  if (!CheckCryptoTokens(pdu, lrq.m_cryptoTokens, H225_LocationRequest::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         lrq.m_tokens, H225_LocationRequest::e_tokens,
+                         lrq.m_cryptoTokens, H225_LocationRequest::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveLocationRequest(lrq);
@@ -1493,7 +1548,8 @@ BOOL H225_RAS::OnReceiveLocationRequest(const H225_LocationRequest & /*lrq*/)
 void H225_RAS::OnSendLocationConfirm(H323RasPDU & pdu, H225_LocationConfirm & lcf)
 {
   OnSendLocationConfirm(lcf);
-  pdu.Prepare(lcf.m_cryptoTokens, lcf, H225_LocationRequest::e_cryptoTokens);
+  pdu.Prepare(lcf.m_tokens, H225_LocationRequest::e_tokens,
+              lcf.m_cryptoTokens, H225_LocationRequest::e_cryptoTokens);
 }
 
 
@@ -1525,7 +1581,8 @@ BOOL H225_RAS::OnReceiveLocationConfirm(const H225_LocationConfirm & /*lcf*/)
 void H225_RAS::OnSendLocationReject(H323RasPDU & pdu, H225_LocationReject & lrj)
 {
   OnSendLocationReject(lrj);
-  pdu.Prepare(lrj.m_cryptoTokens, lrj, H225_LocationReject::e_cryptoTokens);
+  pdu.Prepare(lrj.m_tokens, H225_LocationReject::e_tokens,
+              lrj.m_cryptoTokens, H225_LocationReject::e_cryptoTokens);
 }
 
 
@@ -1539,7 +1596,9 @@ BOOL H225_RAS::OnReceiveLocationReject(const H323RasPDU & pdu, const H225_Locati
   if (!CheckForResponse(H225_RasMessage::e_locationRequest, lrj.m_requestSeqNum, &lrj.m_rejectReason))
     return FALSE;
 
-  if (!CheckCryptoTokens(pdu, lrj.m_cryptoTokens, H225_LocationReject::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         lrj.m_tokens, H225_LocationReject::e_tokens,
+                         lrj.m_cryptoTokens, H225_LocationReject::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveLocationReject(lrj);
@@ -1555,7 +1614,8 @@ BOOL H225_RAS::OnReceiveLocationReject(const H225_LocationReject & /*lrj*/)
 void H225_RAS::OnSendInfoRequest(H323RasPDU & pdu, H225_InfoRequest & irq)
 {
   OnSendInfoRequest(irq);
-  pdu.Prepare(irq.m_cryptoTokens, irq, H225_InfoRequest::e_cryptoTokens);
+  pdu.Prepare(irq.m_tokens, H225_InfoRequest::e_tokens,
+              irq.m_cryptoTokens, H225_InfoRequest::e_cryptoTokens);
 }
 
 
@@ -1566,7 +1626,9 @@ void H225_RAS::OnSendInfoRequest(H225_InfoRequest & /*irq*/)
 
 BOOL H225_RAS::OnReceiveInfoRequest(const H323RasPDU & pdu, const H225_InfoRequest & irq)
 {
-  if (!CheckCryptoTokens(pdu, irq.m_cryptoTokens, H225_InfoRequest::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         irq.m_tokens, H225_InfoRequest::e_tokens,
+                         irq.m_cryptoTokens, H225_InfoRequest::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveInfoRequest(irq);
@@ -1582,7 +1644,8 @@ BOOL H225_RAS::OnReceiveInfoRequest(const H225_InfoRequest & /*irq*/)
 void H225_RAS::OnSendInfoRequestResponse(H323RasPDU & pdu, H225_InfoRequestResponse & irr)
 {
   OnSendInfoRequestResponse(irr);
-  pdu.Prepare(irr.m_cryptoTokens, irr, H225_InfoRequestResponse::e_cryptoTokens);
+  pdu.Prepare(irr.m_tokens, H225_InfoRequestResponse::e_tokens,
+              irr.m_cryptoTokens, H225_InfoRequestResponse::e_cryptoTokens);
 }
 
 
@@ -1596,7 +1659,9 @@ BOOL H225_RAS::OnReceiveInfoRequestResponse(const H323RasPDU & pdu, const H225_I
   if (!CheckForResponse(H225_RasMessage::e_infoRequest, irr.m_requestSeqNum))
     return FALSE;
 
-  if (!CheckCryptoTokens(pdu, irr.m_cryptoTokens, H225_InfoRequestResponse::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         irr.m_tokens, H225_InfoRequestResponse::e_tokens,
+                         irr.m_cryptoTokens, H225_InfoRequestResponse::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveInfoRequestResponse(irr);
@@ -1612,7 +1677,8 @@ BOOL H225_RAS::OnReceiveInfoRequestResponse(const H225_InfoRequestResponse & /*i
 void H225_RAS::OnSendNonStandardMessage(H323RasPDU & pdu, H225_NonStandardMessage & nsm)
 {
   OnSendNonStandardMessage(nsm);
-  pdu.Prepare(nsm.m_cryptoTokens, nsm, H225_InfoRequestResponse::e_cryptoTokens);
+  pdu.Prepare(nsm.m_tokens, H225_InfoRequestResponse::e_tokens,
+              nsm.m_cryptoTokens, H225_InfoRequestResponse::e_cryptoTokens);
 }
 
 
@@ -1623,7 +1689,9 @@ void H225_RAS::OnSendNonStandardMessage(H225_NonStandardMessage & /*nsm*/)
 
 BOOL H225_RAS::OnReceiveNonStandardMessage(const H323RasPDU & pdu, const H225_NonStandardMessage & nsm)
 {
-  if (!CheckCryptoTokens(pdu, nsm.m_cryptoTokens, H225_NonStandardMessage::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         nsm.m_tokens, H225_NonStandardMessage::e_tokens,
+                         nsm.m_cryptoTokens, H225_NonStandardMessage::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveNonStandardMessage(nsm);
@@ -1639,7 +1707,8 @@ BOOL H225_RAS::OnReceiveNonStandardMessage(const H225_NonStandardMessage & /*nsm
 void H225_RAS::OnSendUnknownMessageResponse(H323RasPDU & pdu, H225_UnknownMessageResponse & umr)
 {
   OnSendUnknownMessageResponse(umr);
-  pdu.Prepare(umr.m_cryptoTokens, umr, H225_UnknownMessageResponse::e_cryptoTokens);
+  pdu.Prepare(umr.m_tokens, H225_UnknownMessageResponse::e_tokens,
+              umr.m_cryptoTokens, H225_UnknownMessageResponse::e_cryptoTokens);
 }
 
 
@@ -1650,7 +1719,9 @@ void H225_RAS::OnSendUnknownMessageResponse(H225_UnknownMessageResponse & /*umr*
 
 BOOL H225_RAS::OnReceiveUnknownMessageResponse(const H323RasPDU & pdu, const H225_UnknownMessageResponse & umr)
 {
-  if (!CheckCryptoTokens(pdu, umr.m_cryptoTokens, H225_UnknownMessageResponse::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         umr.m_tokens, H225_UnknownMessageResponse::e_tokens,
+                         umr.m_cryptoTokens, H225_UnknownMessageResponse::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveUnknownMessageResponse(umr);
@@ -1666,7 +1737,8 @@ BOOL H225_RAS::OnReceiveUnknownMessageResponse(const H225_UnknownMessageResponse
 void H225_RAS::OnSendRequestInProgress(H323RasPDU & pdu, H225_RequestInProgress & rip)
 {
   OnSendRequestInProgress(rip);
-  pdu.Prepare(rip.m_cryptoTokens, rip, H225_RequestInProgress::e_cryptoTokens);
+  pdu.Prepare(rip.m_tokens, H225_RequestInProgress::e_tokens,
+              rip.m_cryptoTokens, H225_RequestInProgress::e_cryptoTokens);
 }
 
 
@@ -1678,7 +1750,8 @@ void H225_RAS::OnSendRequestInProgress(H225_RequestInProgress & /*rip*/)
 void H225_RAS::OnSendResourcesAvailableIndicate(H323RasPDU & pdu, H225_ResourcesAvailableIndicate & rai)
 {
   OnSendResourcesAvailableIndicate(rai);
-  pdu.Prepare(rai.m_cryptoTokens, rai, H225_ResourcesAvailableIndicate::e_cryptoTokens);
+  pdu.Prepare(rai.m_tokens, H225_ResourcesAvailableIndicate::e_tokens,
+              rai.m_cryptoTokens, H225_ResourcesAvailableIndicate::e_cryptoTokens);
 }
 
 
@@ -1689,7 +1762,9 @@ void H225_RAS::OnSendResourcesAvailableIndicate(H225_ResourcesAvailableIndicate
 
 BOOL H225_RAS::OnReceiveResourcesAvailableIndicate(const H323RasPDU & pdu, const H225_ResourcesAvailableIndicate & rai)
 {
-  if (!CheckCryptoTokens(pdu, rai.m_cryptoTokens, H225_ResourcesAvailableIndicate::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         rai.m_tokens, H225_ResourcesAvailableIndicate::e_tokens,
+                         rai.m_cryptoTokens, H225_ResourcesAvailableIndicate::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveResourcesAvailableIndicate(rai);
@@ -1705,7 +1780,8 @@ BOOL H225_RAS::OnReceiveResourcesAvailableIndicate(const H225_ResourcesAvailable
 void H225_RAS::OnSendResourcesAvailableConfirm(H323RasPDU & pdu, H225_ResourcesAvailableConfirm & rac)
 {
   OnSendResourcesAvailableConfirm(rac);
-  pdu.Prepare(rac.m_cryptoTokens, rac, H225_ResourcesAvailableConfirm::e_cryptoTokens);
+  pdu.Prepare(rac.m_tokens, H225_ResourcesAvailableConfirm::e_tokens,
+              rac.m_cryptoTokens, H225_ResourcesAvailableConfirm::e_cryptoTokens);
 }
 
 
@@ -1719,7 +1795,9 @@ BOOL H225_RAS::OnReceiveResourcesAvailableConfirm(const H323RasPDU & pdu, const
   if (!CheckForResponse(H225_RasMessage::e_resourcesAvailableIndicate, rac.m_requestSeqNum))
     return FALSE;
 
-  if (!CheckCryptoTokens(pdu, rac.m_cryptoTokens, H225_ResourcesAvailableConfirm::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         rac.m_tokens, H225_ResourcesAvailableConfirm::e_tokens,
+                         rac.m_cryptoTokens, H225_ResourcesAvailableConfirm::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveResourcesAvailableConfirm(rac);
@@ -1735,7 +1813,8 @@ BOOL H225_RAS::OnReceiveResourcesAvailableConfirm(const H225_ResourcesAvailableC
 void H225_RAS::OnSendServiceControlIndication(H323RasPDU & pdu, H225_ServiceControlIndication & sci)
 {
   OnSendServiceControlIndication(sci);
-  pdu.Prepare(sci.m_cryptoTokens, sci, H225_ServiceControlIndication::e_cryptoTokens);
+  pdu.Prepare(sci.m_tokens, H225_ServiceControlIndication::e_tokens,
+              sci.m_cryptoTokens, H225_ServiceControlIndication::e_cryptoTokens);
 }
 
 
@@ -1746,7 +1825,9 @@ void H225_RAS::OnSendServiceControlIndication(H225_ServiceControlIndication & /*
 
 BOOL H225_RAS::OnReceiveServiceControlIndication(const H323RasPDU & pdu, const H225_ServiceControlIndication & sci)
 {
-  if (!CheckCryptoTokens(pdu, sci.m_cryptoTokens, H225_ServiceControlIndication::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         sci.m_tokens, H225_ServiceControlIndication::e_tokens,
+                         sci.m_cryptoTokens, H225_ServiceControlIndication::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveServiceControlIndication(sci);
@@ -1762,7 +1843,8 @@ BOOL H225_RAS::OnReceiveServiceControlIndication(const H225_ServiceControlIndica
 void H225_RAS::OnSendServiceControlResponse(H323RasPDU & pdu, H225_ServiceControlResponse & scr)
 {
   OnSendServiceControlResponse(scr);
-  pdu.Prepare(scr.m_cryptoTokens, scr, H225_ResourcesAvailableConfirm::e_cryptoTokens);
+  pdu.Prepare(scr.m_tokens, H225_ResourcesAvailableConfirm::e_tokens,
+              scr.m_cryptoTokens, H225_ResourcesAvailableConfirm::e_cryptoTokens);
 }
 
 
@@ -1776,7 +1858,9 @@ BOOL H225_RAS::OnReceiveServiceControlResponse(const H323RasPDU & pdu, const H22
   if (!CheckForResponse(H225_RasMessage::e_serviceControlIndication, scr.m_requestSeqNum))
     return FALSE;
 
-  if (!CheckCryptoTokens(pdu, scr.m_cryptoTokens, H225_ServiceControlResponse::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         scr.m_tokens, H225_ServiceControlResponse::e_tokens,
+                         scr.m_cryptoTokens, H225_ServiceControlResponse::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveServiceControlResponse(scr);
@@ -1792,7 +1876,8 @@ BOOL H225_RAS::OnReceiveServiceControlResponse(const H225_ServiceControlResponse
 void H225_RAS::OnSendInfoRequestAck(H323RasPDU & pdu, H225_InfoRequestAck & iack)
 {
   OnSendInfoRequestAck(iack);
-  pdu.Prepare(iack.m_cryptoTokens, iack, H225_InfoRequestAck::e_cryptoTokens);
+  pdu.Prepare(iack.m_tokens, H225_InfoRequestAck::e_tokens,
+              iack.m_cryptoTokens, H225_InfoRequestAck::e_cryptoTokens);
 }
 
 
@@ -1806,7 +1891,9 @@ BOOL H225_RAS::OnReceiveInfoRequestAck(const H323RasPDU & pdu, const H225_InfoRe
   if (!CheckForResponse(H225_RasMessage::e_infoRequestResponse, iack.m_requestSeqNum))
     return FALSE;
 
-  if (!CheckCryptoTokens(pdu, iack.m_cryptoTokens, H225_InfoRequestAck::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         iack.m_tokens, H225_InfoRequestAck::e_tokens,
+                         iack.m_cryptoTokens, H225_InfoRequestAck::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveInfoRequestAck(iack);
@@ -1822,7 +1909,8 @@ BOOL H225_RAS::OnReceiveInfoRequestAck(const H225_InfoRequestAck & /*iack*/)
 void H225_RAS::OnSendInfoRequestNak(H323RasPDU & pdu, H225_InfoRequestNak & inak)
 {
   OnSendInfoRequestNak(inak);
-  pdu.Prepare(inak.m_cryptoTokens, inak, H225_InfoRequestAck::e_cryptoTokens);
+  pdu.Prepare(inak.m_tokens, H225_InfoRequestAck::e_tokens,
+              inak.m_cryptoTokens, H225_InfoRequestAck::e_cryptoTokens);
 }
 
 
@@ -1836,7 +1924,9 @@ BOOL H225_RAS::OnReceiveInfoRequestNak(const H323RasPDU & pdu, const H225_InfoRe
   if (!CheckForResponse(H225_RasMessage::e_infoRequestResponse, inak.m_requestSeqNum, &inak.m_nakReason))
     return FALSE;
 
-  if (!CheckCryptoTokens(pdu, inak.m_cryptoTokens, H225_InfoRequestNak::e_cryptoTokens))
+  if (!CheckCryptoTokens(pdu,
+                         inak.m_tokens, H225_InfoRequestNak::e_tokens,
+                         inak.m_cryptoTokens, H225_InfoRequestNak::e_cryptoTokens))
     return FALSE;
 
   return OnReceiveInfoRequestNak(inak);
@@ -1849,7 +1939,7 @@ BOOL H225_RAS::OnReceiveInfoRequestNak(const H225_InfoRequestNak & /*inak*/)
 }
 
 
-BOOL H225_RAS::OnReceiveUnkown(const H323RasPDU &)
+BOOL H225_RAS::OnReceiveUnknown(const H323RasPDU &)
 {
   H323RasPDU response;
   response.BuildUnknownMessageResponse(0);
@@ -1857,19 +1947,11 @@ BOOL H225_RAS::OnReceiveUnkown(const H323RasPDU &)
 }
 
 
-unsigned H225_RAS::GetNextSequenceNumber()
-{
-  PWaitAndSignal mutex(nextSequenceNumberMutex);
-  nextSequenceNumber++;
-  if (nextSequenceNumber >= 65536)
-    nextSequenceNumber = 1;
-  return nextSequenceNumber;
-}
-
-
 BOOL H225_RAS::CheckCryptoTokens(const H323RasPDU & pdu,
+                                 const H225_ArrayOf_ClearToken & clearTokens,
+                                 unsigned clearOptionalField,
                                  const H225_ArrayOf_CryptoH323Token & cryptoTokens,
-                                 unsigned optionalField)
+                                 unsigned cryptoOptionalField)
 {
   // If cypto token checking disabled, just return TRUE.
   if (!GetCheckResponseCryptoTokens())
@@ -1878,7 +1960,8 @@ BOOL H225_RAS::CheckCryptoTokens(const H323RasPDU & pdu,
   if (lastRequest != NULL && pdu.GetAuthenticators().IsEmpty())
     ((H323RasPDU &)pdu).SetAuthenticators(lastRequest->requestPDU.GetAuthenticators());
 
-  if (pdu.Validate(cryptoTokens, (const PASN_Sequence &)pdu.GetObject(), optionalField))
+  if (pdu.Validate(clearTokens, clearOptionalField,
+                   cryptoTokens, cryptoOptionalField) == H235Authenticator::e_OK)
     return TRUE;
 
   /* Note that a crypto tokens error is flagged to the requestor in the
