@@ -27,6 +27,22 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h323caps.cxx,v $
+ * Revision 1.61  2002/12/05 12:29:31  rogerh
+ * Add non standard codec identifier for Xiph.org
+ *
+ * Revision 1.60  2002/11/27 11:47:09  robertj
+ * Fixed GNU warning
+ *
+ * Revision 1.59  2002/11/26 22:48:18  craigs
+ * Changed nonStd codec identification to use a table for MS codecs
+ *
+ * Revision 1.58  2002/11/26 13:52:59  craigs
+ * Added PrintOn function for outputting names of nonStandard codecs
+ *
+ * Revision 1.57  2002/11/09 04:44:24  robertj
+ * Fixed function to add capabilities to return correct table entry index.
+ * Cosmetic changes.
+ *
  * Revision 1.56  2002/08/05 10:03:47  robertj
  * Cosmetic changes to normalise the usage of pragma interface/implementation.
  *
@@ -251,6 +267,25 @@ static H323CapabilityRegistration * RegisteredCapabilitiesListHead;
 #define new PNEW
 
 
+#if PTRACING
+ostream & operator<<(ostream & o , H323Capability::MainTypes t)
+{
+  const char * const names[] = {
+    "Audio", "Video", "Data", "UserInput"
+  };
+  return o << names[t];
+}
+
+ostream & operator<<(ostream & o , H323Capability::CapabilityDirection d)
+{
+  const char * const names[] = {
+    "Unknown", "Receive", "Transmit", "ReceiveAndTransmit", "NoDirection"
+  };
+  return o << names[d];
+}
+#endif
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 H323Capability::H323Capability()
@@ -376,25 +411,6 @@ BOOL H323Capability::IsUsable(const H323Connection &) const
 }
 
 
-#if PTRACING
-ostream & operator<<(ostream & o , H323Capability::MainTypes t)
-{
-  const char * const names[] = {
-    "Audio", "Video", "Data", "UserInput"
-  };
-  return o << names[t];
-}
-
-ostream & operator<<(ostream & o , H323Capability::CapabilityDirection d)
-{
-  const char * const names[] = {
-    "Unknown", "Receive", "Transmit", "ReceiveAndTransmit", "NoDirection"
-  };
-  return o << names[d];
-}
-#endif
-
-
 /////////////////////////////////////////////////////////////////////////////
 
 H323Channel * H323RealTimeCapability::CreateChannel(H323Connection & connection,
@@ -1148,6 +1164,16 @@ PObject * H323_G711Capability::Clone() const
 }
 
 
+unsigned H323_G711Capability::GetSubType() const
+{
+  static const unsigned G711SubType[2][2] = {
+    { H245_AudioCapability::e_g711Alaw64k, H245_AudioCapability::e_g711Alaw56k },
+    { H245_AudioCapability::e_g711Ulaw64k, H245_AudioCapability::e_g711Ulaw56k }
+  };
+  return G711SubType[mode][speed];
+}
+
+
 PString H323_G711Capability::GetFormatName() const
 {
   static const char * const G711Name[2][2] = {
@@ -1158,18 +1184,6 @@ PString H323_G711Capability::GetFormatName() const
 }
 
 
-static const unsigned G711SubType[2][2] = {
-  { H245_AudioCapability::e_g711Alaw64k, H245_AudioCapability::e_g711Alaw56k },
-  { H245_AudioCapability::e_g711Ulaw64k, H245_AudioCapability::e_g711Ulaw56k }
-};
-
-
-unsigned H323_G711Capability::GetSubType() const
-{
-  return G711SubType[mode][speed];
-}
-
-
 H323Codec * H323_G711Capability::CreateCodec(H323Codec::Direction direction) const
 {
   unsigned packetSize = 8*(direction == H323Codec::Encoder ? txFramesInPacket : rxFramesInPacket);
@@ -1217,12 +1231,6 @@ PObject * H323_UserInputCapability::Clone() const
 }
 
 
-PString H323_UserInputCapability::GetFormatName() const
-{
-  return SubTypeNames[subType];
-}
-
-
 H323Capability::MainTypes H323_UserInputCapability::GetMainType() const
 {
   return e_UserInput;
@@ -1246,6 +1254,12 @@ unsigned  H323_UserInputCapability::GetSubType()  const
 }
 
 
+PString H323_UserInputCapability::GetFormatName() const
+{
+  return SubTypeNames[subType];
+}
+
+
 H323Channel * H323_UserInputCapability::CreateChannel(H323Connection &,
                                                       H323Channel::Directions,
                                                       unsigned,
@@ -1541,7 +1555,7 @@ PINDEX H323Capabilities::AddAllCapabilities(H323EndPoint & ep,
                                             PINDEX simultaneous,
                                             const PString & name)
 {
-  PINDEX reply = P_MAX_INDEX;
+  PINDEX reply = descriptorNum == P_MAX_INDEX ? P_MAX_INDEX : simultaneous;
 
   PStringArray wildcard = name.Tokenise('*', FALSE);
 
@@ -1555,7 +1569,7 @@ PINDEX H323Capabilities::AddAllCapabilities(H323EndPoint & ep,
         simultaneous = P_MAX_INDEX;
       }
       else if (simultaneous == P_MAX_INDEX) {
-        if (reply != P_MAX_INDEX)
+        if (reply == P_MAX_INDEX)
           reply = num;
         simultaneous = num;
       }
@@ -2093,3 +2107,105 @@ H323CapabilityRegistration::H323CapabilityRegistration(const char * name)
 
 
 /////////////////////////////////////////////////////////////////////////////
+
+#ifndef PASN_NOPRINTON
+
+
+struct msNonStandardCodecDef {
+  char * name;
+  BYTE sig[2];
+};
+
+
+static msNonStandardCodecDef msNonStandardCodec[] = {
+  { "L&H CELP 4.8k", { 0x01, 0x11 } },
+  { "ADPCM",         { 0x02, 0x00 } },
+  { "L&H CELP 8k",   { 0x02, 0x11 } },
+  { "L&H CELP 12k",  { 0x03, 0x11 } },
+  { "L&H CELP 16k",  { 0x04, 0x11 } },
+  { "IMA-ADPCM",     { 0x11, 0x00 } },
+  { "GSM",           { 0x31, 0x00 } },
+  { NULL,            { 0,    0    } }
+};
+
+void H245_AudioCapability::PrintOn(ostream & strm) const
+{
+  strm << GetTagName();
+
+  if (tag == 0) {
+    H245_NonStandardParameter & param = (H245_NonStandardParameter &)GetObject();
+    const PBYTEArray & data = param.m_data;
+
+    switch (param.m_nonStandardIdentifier.GetTag()) {
+      case H245_NonStandardIdentifier::e_h221NonStandard:
+        {
+          H245_NonStandardIdentifier_h221NonStandard & h221 = param.m_nonStandardIdentifier;
+
+          // Microsoft is 181/0/21324
+          if ((h221.m_t35CountryCode   == 181) &&
+              (h221.m_t35Extension     == 0) &&
+              (h221.m_manufacturerCode == 21324)
+            ) {
+            PString name = "Unknown";
+            PINDEX i;
+            if (data.GetSize() >= 21) {
+              for (i = 0; msNonStandardCodec[i].name != NULL; i++) {
+                if ((data[20] == msNonStandardCodec[i].sig[0]) && 
+                    (data[21] == msNonStandardCodec[i].sig[1])) {
+                  name = msNonStandardCodec[i].name;
+                  break;
+                }
+              }
+            }
+            strm << (PString(" [Microsoft") & name) << "]";
+          }
+
+          // Equivalence is 9/0/61
+          else if ((h221.m_t35CountryCode   == 9) &&
+                   (h221.m_t35Extension     == 0) &&
+                   (h221.m_manufacturerCode == 61)
+                  ) {
+            PString name;
+            if (data.GetSize() > 0)
+              name = PString((const char *)(const BYTE *)data, data.GetSize());
+            strm << " [Equivalence " << name << "]";
+          }
+
+          // Xiph is 181/0/38
+          else if ((h221.m_t35CountryCode   == 181) &&
+                   (h221.m_t35Extension     == 0) &&
+                   (h221.m_manufacturerCode == 38)
+                  ) {
+            PString name;
+            if (data.GetSize() > 0)
+              name = PString((const char *)(const BYTE *)data, data.GetSize());
+            strm << " [Xiph " << name << "]";
+          }
+
+          // Cisco is 181/0/18
+          else if ((h221.m_t35CountryCode   == 181) &&
+                   (h221.m_t35Extension     == 0) &&
+                   (h221.m_manufacturerCode == 18)
+                  ) {
+            PString name;
+            if (data.GetSize() > 0)
+              name = PString((const char *)(const BYTE *)data, data.GetSize());
+            strm << " [Cisco " << name << "]";
+          }
+
+        }
+        break;
+      default:
+        break;
+    }
+  }
+
+  if (choice == NULL)
+    strm << " (NULL)";
+  else {
+    strm << ' ' << *choice;
+  }
+
+  //PASN_Choice::PrintOn(strm);
+}
+#endif
