@@ -27,6 +27,38 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h323ep.cxx,v $
+ * Revision 1.159  2003/03/04 03:58:32  robertj
+ * Fixed missing local interface usage if specified in UseGatekeeper()
+ *
+ * Revision 1.158  2003/02/28 09:00:37  rogerh
+ * remove redundant code
+ *
+ * Revision 1.157  2003/02/25 23:51:49  robertj
+ * Fxied bug where not getting last port in range, thanks Sonya Cooper-Hull
+ *
+ * Revision 1.156  2003/02/09 00:48:09  robertj
+ * Added function to return if registered with gatekeeper.
+ *
+ * Revision 1.155  2003/02/05 06:32:10  robertj
+ * Fixed non-stun symmetric NAT support recently broken.
+ *
+ * Revision 1.154  2003/02/05 04:56:35  robertj
+ * Fixed setting STUN server to enpty string clearing stun variable.
+ *
+ * Revision 1.153  2003/02/04 07:06:41  robertj
+ * Added STUN support.
+ *
+ * Revision 1.152  2003/02/01 13:31:22  robertj
+ * Changes to support CAT authentication in RAS.
+ *
+ * Revision 1.151  2003/01/26 05:57:29  robertj
+ * Changed ParsePartyName so will accept addresses of the form
+ *   alias@gk:address which will do an LRQ call to "address" using "alias"
+ *   to determine the IP address to connect to.
+ *
+ * Revision 1.150  2003/01/23 02:36:32  robertj
+ * Increased (and made configurable) timeout for H.245 channel TCP connection.
+ *
  * Revision 1.149  2003/01/06 06:13:37  robertj
  * Increased maximum possible jitter configuration to 10 seconds.
  *
@@ -537,6 +569,7 @@
 #include "../version.h"
 
 #include <ptclib/random.h>
+#include <ptclib/pstun.h>
 
 
 #ifndef IPTOS_PREC_CRITIC_ECP
@@ -711,6 +744,7 @@ H323EndPoint::H323EndPoint()
   : soundChannelPlayDevice(PSoundChannel::GetDefaultDevice(PSoundChannel::Player)),
     soundChannelRecordDevice(PSoundChannel::GetDefaultDevice(PSoundChannel::Recorder)),
     signallingChannelCallTimeout(0, 0, 1),  // Minutes
+    controlChannelStartTimeout(0, 0, 2),    // Minutes
     endSessionTimeout(0, 10),               // Seconds
     masterSlaveDeterminationTimeout(0, 30), // Seconds
     capabilityExchangeTimeout(0, 30),       // Seconds
@@ -767,6 +801,8 @@ H323EndPoint::H323EndPoint()
   tcpPorts.current = tcpPorts.base = tcpPorts.max = 0;
   udpPorts.current = udpPorts.base = udpPorts.max = 0;
 
+  stun = NULL;
+
 #ifdef _WIN32
   // Windows MultMedia stuff seems to need greater depth due to enormous
   // latencies in its operation, need to use DirectSound maybe?
@@ -819,6 +855,8 @@ H323EndPoint::~H323EndPoint()
   // Clean up any connections that the cleaner thread missed
   CleanUpConnections();
 
+  delete stun;
+
   PTRACE(3, "H323\tDeleted endpoint.");
 }
 
@@ -960,17 +998,26 @@ BOOL H323EndPoint::UseGatekeeper(const PString & address,
     }
   }
 
+  H323Transport * transport = NULL;
+  if (!localAddress.IsEmpty()) {
+    H323TransportAddress iface(localAddress);
+    PIPSocket::Address ip;
+    WORD port = H225_RAS::DefaultRasUdpPort;
+    if (iface.GetIpAndPort(ip, port))
+      transport = new H323TransportUDP(*this, ip, port);
+  }
+
   if (address.IsEmpty()) {
     if (identifier.IsEmpty())
-      return DiscoverGatekeeper();
+      return DiscoverGatekeeper(transport);
     else
-      return LocateGatekeeper(identifier);
+      return LocateGatekeeper(identifier, transport);
   }
   else {
     if (identifier.IsEmpty())
-      return SetGatekeeper(address);
+      return SetGatekeeper(address, transport);
     else
-      return SetGatekeeperZone(address, identifier);
+      return SetGatekeeperZone(address, identifier, transport);
   }
 }
 
@@ -1044,6 +1091,15 @@ H323Gatekeeper * H323EndPoint::CreateGatekeeper(H323Transport * transport)
 }
 
 
+BOOL H323EndPoint::IsRegisteredWithGatekeeper() const
+{
+  if (gatekeeper == NULL)
+    return FALSE;
+
+  return gatekeeper->IsRegistered();
+}
+
+
 BOOL H323EndPoint::RemoveGatekeeper(int reason)
 {
   BOOL ok = TRUE;
@@ -1084,6 +1140,7 @@ H235Authenticators H323EndPoint::CreateAuthenticators()
   authenticators.Append(new H235AuthProcedure1);
 #endif
   authenticators.Append(new H235AuthSimpleMD5);
+  authenticators.Append(new H235AuthCAT);
 
   return authenticators;
 }
@@ -1233,7 +1290,10 @@ H323Connection * H323EndPoint::InternalMakeCall(const PString & trasferFromToken
 
   PString alias;
   H323TransportAddress address;
-  ParsePartyName(remoteParty, alias, address);
+  if (!ParsePartyName(remoteParty, alias, address)) {
+    PTRACE(2, "H323\tCould not parse \"" << remoteParty << '"');
+    return NULL;
+  }
 
   if (transport == NULL) {
     // Restriction: the call must be made on the same transport as the one
@@ -1303,9 +1363,9 @@ H323Connection * H323EndPoint::InternalMakeCall(const PString & trasferFromToken
 }
 
 
-void H323EndPoint::ParsePartyName(const PString & remoteParty,
+BOOL H323EndPoint::ParsePartyName(const PString & remoteParty,
                                   PString & alias,
-                                  H323TransportAddress & address) const
+                                  H323TransportAddress & address)
 {
   /*Determine the alias part and the address part of the remote party name
     string. This depends on if there is an '@' to separate the alias from the
@@ -1314,14 +1374,28 @@ void H323EndPoint::ParsePartyName(const PString & remoteParty,
   PINDEX at = remoteParty.FindLast('@');
   if (at != P_MAX_INDEX) {
     alias = remoteParty.Left(at++);
-    address = remoteParty.Mid(at);
-    return;
+    if (remoteParty.NumCompare("gk:", 3, at) != EqualTo)
+      address = remoteParty.Mid(at);
+    else {
+      H323TransportAddress gkAddr = remoteParty.Mid(at+3);
+      PTRACE(3, "H323\tLooking for \"" << alias << "\" on gatekeeper at " << gkAddr);
+
+      H323Gatekeeper * gk = CreateGatekeeper(new H323TransportUDP(*this));
+      if (gk->DiscoverByAddress(gkAddr) && gk->LocationRequest(alias, address)) {
+        PTRACE(3, "H323\tLocation Request of \"" << alias << "\" on gk " << gkAddr << " found " << address);
+      }
+      else {
+        PTRACE(2, "H323\tLocation Request failed for \"" << alias << "\" on gk " << gkAddr);
+      }
+      delete gk;
+    }
+    return !address.IsEmpty();
   }
 
   if (gatekeeper == NULL) {
     alias = PString::Empty();
     address = remoteParty;
-    return;
+    return !address.IsEmpty();
   }
 
   if (remoteParty.FindOneOf("$.:") != P_MAX_INDEX) {
@@ -1329,12 +1403,13 @@ void H323EndPoint::ParsePartyName(const PString & remoteParty,
     PIPSocket::Address ip;
     if (address.GetIpAddress(ip) && ip.IsValid()) {
       alias = PString::Empty();
-      return;
+      return TRUE;
     }
   }
 
   alias = remoteParty;
   address = PString::Empty();
+  return TRUE;
 }
 
 
@@ -2110,6 +2185,55 @@ void H323EndPoint::SetAudioJitterDelay(unsigned minDelay, unsigned maxDelay)
 }
 
 
+PSTUNClient * H323EndPoint::GetSTUN(const PIPSocket::Address & ip) const
+{
+  if (ip.IsValid() && IsLocalAddress(ip))
+    return NULL;
+
+  return stun;
+}
+
+
+void H323EndPoint::SetSTUNServer(const PString & server)
+{
+  delete stun;
+
+  if (server.IsEmpty())
+    stun = NULL;
+  else {
+    stun = new PSTUNClient(server,
+                           GetUDPPortBase(), GetUDPPortMax(),
+                           GetRtpIpPortBase(), GetRtpIpPortMax());
+    PTRACE(2, "H323\tSTUN server \"" << server << "\" replies " << stun->GetNatTypeName());
+  }
+}
+
+
+BOOL H323EndPoint::IsLocalAddress(const PIPSocket::Address & ip) const
+{
+  /* Check if the remote address is a private IP address.
+   * RFC 1918 specifies the following private IP addresses
+   * 10.0.0.0    - 10.255.255.255.255
+   * 172.16.0.0  - 172.31.255.255
+   * 192.168.0.0 - 192.168.255.255
+   */
+
+  return (ip.Byte1() == 10)
+         ||
+         (
+           (ip.Byte1() == 172)
+           &&
+           (ip.Byte2() >= 16) && (ip.Byte2() <= 31)
+         )
+         ||
+         (
+           (ip.Byte1() == 192) 
+           &&
+           (ip.Byte2() == 168)
+         );
+}
+
+
 void H323EndPoint::PortInfo::Set(unsigned newBase,
                                  unsigned newMax,
                                  unsigned range,
@@ -2146,7 +2270,7 @@ WORD H323EndPoint::PortInfo::GetNext(unsigned increment)
 {
   PWaitAndSignal m(mutex);
 
-  if (current < base || current >= (max-increment))
+  if (current < base || current > (max-increment))
     current = base;
 
   if (current == 0)
