@@ -27,6 +27,22 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h323pdu.cxx,v $
+ * Revision 1.123  2003/02/25 06:48:19  robertj
+ * More work on PDU transaction abstraction.
+ *
+ * Revision 1.122  2003/02/21 05:25:45  craigs
+ * Abstracted out underlying transports for use with peerelements
+ *
+ * Revision 1.121  2003/02/03 04:31:05  robertj
+ * Added special case for Cisco vendor field, they leave it rather incomplete,
+ *
+ * Revision 1.120  2003/01/26 02:57:58  craigs
+ * Fixed oops in last checkin
+ *
+ * Revision 1.119  2003/01/26 02:50:38  craigs
+ * Change so SETUP PDU uses conference and callIdentifier from H323Connection,
+ * rather than both doing seperately and then overwriting
+ *
  * Revision 1.118  2002/11/28 04:41:48  robertj
  * Added support for RAS ServiceControlIndication command.
  *
@@ -643,9 +659,20 @@ PString H323GetApplicationInfo(const H225_VendorIdentifier & vendor)
 {
   PStringStream str;
 
-  str << vendor.m_productId.AsString() << '\t'
-      << vendor.m_versionId.AsString() << '\t'
-      << vendor.m_vendor.m_t35CountryCode;
+  PString product = vendor.m_productId.AsString();
+  PString version = vendor.m_versionId.AsString();
+
+  // Special case, Cisco IOS does not put in the product and version fields
+  if (vendor.m_vendor.m_t35CountryCode == 181 &&
+      vendor.m_vendor.m_t35Extension == 0 &&
+      vendor.m_vendor.m_manufacturerCode == 18) {
+    if (product.IsEmpty())
+      product = "Cisco IOS";
+    if (version.IsEmpty())
+      version = "12.2";
+  }
+
+  str << product << '\t' << version << '\t' << vendor.m_vendor.m_t35CountryCode;
   if (vendor.m_vendor.m_t35Extension != 0)
     str << '.' << vendor.m_vendor.m_t35Extension;
   str << '/' << vendor.m_vendor.m_manufacturerCode;
@@ -690,11 +717,11 @@ H225_Setup_UUIE & H323SignalPDU::BuildSetup(const H323Connection & connection,
   setup.IncludeOptionalField(H225_Setup_UUIE::e_sourceAddress);
   H323SetAliasAddresses(endpoint.GetAliasNames(), setup.m_sourceAddress);
 
-  setup.m_conferenceID = OpalGloballyUniqueID();
+  setup.m_conferenceID = connection.GetConferenceIdentifier();
   setup.m_conferenceGoal.SetTag(H225_Setup_UUIE_conferenceGoal::e_create);
   setup.m_callType.SetTag(H225_CallType::e_pointToPoint);
 
-  setup.m_callIdentifier.m_guid = OpalGloballyUniqueID();
+  setup.m_callIdentifier.m_guid = connection.GetCallIdentifier();
   setup.m_mediaWaitForConnect = FALSE;
   setup.m_canOverlapSend = FALSE;
 
@@ -1691,7 +1718,7 @@ H323RasPDU::H323RasPDU()
 
 
 H323RasPDU::H323RasPDU(const H235Authenticators & auth)
-  : authenticators(auth)
+  : H323TransactionPDU(auth)
 {
 }
 
@@ -2089,49 +2116,4 @@ unsigned H323RasPDU::GetSequenceNumber() const
 }
 
 
-BOOL H323RasPDU::Read(H323Transport & transport)
-{
-  if (!transport.ReadPDU(rawPDU)) {
-    PTRACE(1, "H225RAS\tRead error ("
-           << transport.GetErrorNumber(PChannel::LastReadError)
-           << "): " << transport.GetErrorText(PChannel::LastReadError));
-    return FALSE;
-  }
-
-  rawPDU.ResetDecoder();
-  BOOL ok = Decode(rawPDU);
-  if (!ok) {
-    PTRACE(1, "H225RAS\tRead error: PER decode failure:\n  "
-           << setprecision(2) << rawPDU << "\n "  << setprecision(2) << *this);
-    SetTag(UINT_MAX);
-    return TRUE;
-  }
-
-  H323TraceDumpPDU("H225RAS", FALSE, rawPDU, *this, *this, GetSequenceNumber());
-  return TRUE;
-}
-
-
-BOOL H323RasPDU::Write(H323Transport & transport)
-{
-  PPER_Stream strm;
-  Encode(strm);
-  strm.CompleteEncoding();
-
-  // Finalise the security if present
-  for (PINDEX i = 0; i < authenticators.GetSize(); i++)
-    authenticators[i].Finalise(strm);
-
-  H323TraceDumpPDU("H225RAS", TRUE, strm, *this, *this, GetSequenceNumber());
-
-  if (transport.WritePDU(strm))
-    return TRUE;
-
-  PTRACE(1, "H225RAS\tWrite PDU failed ("
-         << transport.GetErrorNumber(PChannel::LastWriteError)
-         << "): " << transport.GetErrorText(PChannel::LastWriteError));
-  return FALSE;
-}
-
-
 /////////////////////////////////////////////////////////////////////////////
