@@ -24,6 +24,9 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h450pdu.cxx,v $
+ * Revision 1.19  2002/11/21 22:37:47  robertj
+ * Fixed problems with unrecognized invoke APDU, thanks Andrea Bondavalli
+ *
  * Revision 1.18  2002/10/01 06:47:39  robertj
  * Fixed GNU compiler warning
  *
@@ -454,6 +457,8 @@ BOOL result = TRUE;
       continue;
     }
 
+    H4501_InterpretationApdu & interpretation = supplementaryService.m_interpretationApdu;
+
     if (supplementaryService.m_serviceApdu.GetTag() == H4501_ServiceApdus::e_rosApdus) {
       H4501_ArrayOf_ROS& operations = (H4501_ArrayOf_ROS&) supplementaryService.m_serviceApdu;
 
@@ -464,7 +469,7 @@ BOOL result = TRUE;
 
         switch (operation.GetTag()) {
           case X880_ROS::e_invoke:
-            result = OnReceivedInvoke((X880_Invoke &)operation);
+            result = OnReceivedInvoke((X880_Invoke &)operation, interpretation);
             break;
 
           case X880_ROS::e_returnResult:
@@ -488,8 +493,7 @@ BOOL result = TRUE;
   return result;
 }
 
-
-BOOL H450xDispatcher::OnReceivedInvoke(X880_Invoke & invoke)
+BOOL H450xDispatcher::OnReceivedInvoke(X880_Invoke & invoke, H4501_InterpretationApdu & interpretation)
 {
   BOOL result = TRUE;
   // Get the invokeId
@@ -511,17 +515,21 @@ BOOL H450xDispatcher::OnReceivedInvoke(X880_Invoke & invoke)
   if (invoke.m_opcode.GetTag() == X880_Code::e_local) {
     int opcode = ((PASN_Integer&) invoke.m_opcode).GetValue();
     if (!opcodeHandler.Contains(opcode)) {
-      PTRACE(2, "H4501\tInvoke of unsupported local opcode:\n  " << invoke);
-      SendInvokeReject(invokeId, 1 /*X880_InvokeProblem::e_unrecognisedOperation*/);
-      result = FALSE;
+      PTRACE(2, "H4501\tInvoke of unsupported local opcode:\n  " << invoke);	  
+      if (interpretation.GetTag() != H4501_InterpretationApdu::e_discardAnyUnrecognizedInvokePdu)
+        SendInvokeReject(invokeId, 1 /*X880_InvokeProblem::e_unrecognisedOperation*/);
+      if (interpretation.GetTag() == H4501_InterpretationApdu::e_clearCallIfAnyInvokePduNotRecognized)
+        result = FALSE;
     }
     else
       result = opcodeHandler[opcode].OnReceivedInvoke(opcode, invokeId, linkedId, argument);
   }
   else {
-    SendInvokeReject(invokeId, 1 /*X880_InvokeProblem::e_unrecognisedOperation*/);
+    if (interpretation.GetTag() != H4501_InterpretationApdu::e_discardAnyUnrecognizedInvokePdu)
+      SendInvokeReject(invokeId, 1 /*X880_InvokeProblem::e_unrecognisedOperation*/);
     PTRACE(2, "H4501\tInvoke of unsupported global opcode:\n  " << invoke);
-    result = FALSE;
+    if (interpretation.GetTag() == H4501_InterpretationApdu::e_clearCallIfAnyInvokePduNotRecognized)
+      result = FALSE;
   }
   return result;
 }
