@@ -24,6 +24,15 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: lid.cxx,v $
+ * Revision 1.93  2003/03/05 06:26:44  robertj
+ * Added function to play a WAV file to LID, thanks Pietro Ravasio
+ *
+ * Revision 1.92  2003/02/13 23:33:36  dereks
+ * Fix reporting of tonenames.
+ *
+ * Revision 1.91  2003/01/29 23:58:17  dereks
+ * Fix typo in United Kingdom tone definition.
+ *
  * Revision 1.90  2002/12/02 03:06:26  robertj
  * Fixed over zealous removal of code when NO_AUDIO_CODECS set.
  *
@@ -350,12 +359,12 @@ static const char * const CallProgressTonesNames[] = {
 
 ostream & operator<<(ostream & o, OpalLineInterfaceDevice::CallProgressTones t)
 {
-  PINDEX i = 0;
-  while ((1 << i) < t)
+  PINDEX i = 0;    
+  while ((1 << i) != t)
     i++;
 
   if (i < PARRAYSIZE(CallProgressTonesNames))
-    return o << CallProgressTonesNames[t];
+    return o << CallProgressTonesNames[i];
   else
     return o << "Unknown";
 }
@@ -860,6 +869,20 @@ BOOL OpalLineInterfaceDevice::StopTone(unsigned)
 }
 
 
+BOOL OpalLineInterfaceDevice::PlayAudio(unsigned /*line*/, const PString & /*filename*/)
+{
+  PTRACE(3, "LID\tBase Class PlayAudio method called, exiting with FALSE");
+  return FALSE;
+}
+
+
+BOOL OpalLineInterfaceDevice::StopAudio(unsigned /*line*/)
+{
+  PTRACE(3, "LID\tBase Class StopAudio method called, exiting with FALSE");
+  return FALSE;
+}
+	
+
 OpalLineInterfaceDevice::CallProgressTones
                 OpalLineInterfaceDevice::DialOut(unsigned line,
                                                  const PString & number,
@@ -1111,7 +1134,7 @@ static struct {
   { "UG", 256,  OpalLineInterfaceDevice::Uganda,                "Uganda" },
   { "UA", 380,  OpalLineInterfaceDevice::Ukraine,               "Ukraine" },
   { "AE", 971,  OpalLineInterfaceDevice::UnitedArabEmirates,    "United Arab Emirates" },
-  { "GB", 44,   OpalLineInterfaceDevice::UnitedKingdom,         "United Kingdom"        "350-440:0.1", "400-450:0.4-0.2-0.4-2", "400:0.375-0.375" },
+  { "GB", 44,   OpalLineInterfaceDevice::UnitedKingdom,         "United Kingdom",       "350-440:0.1", "400-450:0.4-0.2-0.4-2", "400:0.375-0.375" },
   { "US", 1,    OpalLineInterfaceDevice::UnitedStates,          "United States",        "350-440:0.1", "440-480:2.0-4.0",       "480-620:0.5-0.5" },
   { "UY", 598,  OpalLineInterfaceDevice::Uruguay,               "Uruguay" },
   { "VU", 678,  OpalLineInterfaceDevice::Vanuatu,               "Vanuatu" },
