@@ -27,6 +27,20 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: rtp.cxx,v $
+ * Revision 1.87  2003/02/07 00:30:21  robertj
+ * Changes for bizarre usage of RTP code outside of scope of H.323 specs.
+ *
+ * Revision 1.86  2003/02/04 23:50:06  robertj
+ * Changed trace log for RTP session creation to show local address.
+ *
+ * Revision 1.85  2003/02/04 07:06:42  robertj
+ * Added STUN support.
+ *
+ * Revision 1.84  2003/01/07 06:32:22  robertj
+ * Fixed faint possibility of getting an error on UDP write caused by ICMP
+ *   unreachable being received when no UDP read is active on the socket. Then
+ *   the UDP write gets the error stopping transmission.
+ *
  * Revision 1.83  2002/11/19 01:48:00  robertj
  * Allowed get/set of canonical anme and tool name.
  *
@@ -308,6 +322,7 @@
 
 #include "jitter.h"
 #include <ptclib/random.h>
+#include <ptclib/pstun.h>
 
 
 #define new PNEW
@@ -323,7 +338,6 @@
 #define PTRACE_type
 #define PTRACE_subtype
 #define PTRACE_size
-#define PTRACE_name
 #define PTRACE_port
 #endif
 
@@ -1371,6 +1385,8 @@ RTP_UDP::RTP_UDP(unsigned id)
   remoteControlPort = 0;
   shutdownRead = FALSE;
   shutdownWrite = FALSE;
+  dataSocket = NULL;
+  controlSocket = NULL;
 }
 
 
@@ -1378,10 +1394,16 @@ RTP_UDP::~RTP_UDP()
 {
   Close(TRUE);
   Close(FALSE);
+
+  delete dataSocket;
+  delete controlSocket;
 }
 
 
-BOOL RTP_UDP::Open(PIPSocket::Address _localAddress, WORD portBase, WORD portMax, BYTE tos)
+BOOL RTP_UDP::Open(PIPSocket::Address _localAddress,
+                   WORD portBase, WORD portMax,
+                   BYTE tos,
+                   PSTUNClient * stun)
 {
   // save local address 
   localAddress = _localAddress;
@@ -1389,29 +1411,47 @@ BOOL RTP_UDP::Open(PIPSocket::Address _localAddress, WORD portBase, WORD portMax
   localDataPort    = (WORD)(portBase&0xfffe);
   localControlPort = (WORD)(localDataPort + 1);
 
-  while (!dataSocket.Listen(localAddress,    1, localDataPort) ||
-         !controlSocket.Listen(localAddress, 1, localControlPort)) {
-    dataSocket.Close();
-    controlSocket.Close();
-    if ((localDataPort > portMax) || (localDataPort > 0xfffd))
-      return FALSE; // If it ever gets to here the OS has some SERIOUS problems!
-    localDataPort    += 2;
-    localControlPort += 2;
+  delete dataSocket;
+  delete controlSocket;
+  dataSocket = NULL;
+  controlSocket = NULL;
+
+  if (stun != NULL) {
+    if (stun->CreateSocketPair(dataSocket, controlSocket)) {
+      dataSocket->GetLocalAddress(localAddress, localDataPort);
+      controlSocket->GetLocalAddress(localAddress, localControlPort);
+    }
+    else
+      PTRACE(1, "RTP\tSTUN could not create socket pair!");
+  }
+
+  if (dataSocket == NULL || controlSocket == NULL) {
+    dataSocket = new PUDPSocket;
+    controlSocket = new PUDPSocket;
+    while (!dataSocket->Listen(localAddress,    1, localDataPort) ||
+           !controlSocket->Listen(localAddress, 1, localControlPort)) {
+      dataSocket->Close();
+      controlSocket->Close();
+      if ((localDataPort > portMax) || (localDataPort > 0xfffd))
+        return FALSE; // If it ever gets to here the OS has some SERIOUS problems!
+      localDataPort    += 2;
+      localControlPort += 2;
+    }
   }
 
 #ifndef __BEOS__
 
   // Set the IP Type Of Service field for prioritisation of media UDP packets
   // through some Cisco routers and Linux boxes
-  if (!dataSocket.SetOption(IP_TOS, tos, IPPROTO_IP)) {
-    PTRACE(1, "RTP_UDP\tCould not set TOS field in IP header: " << dataSocket.GetErrorText());
+  if (!dataSocket->SetOption(IP_TOS, tos, IPPROTO_IP)) {
+    PTRACE(1, "RTP_UDP\tCould not set TOS field in IP header: " << dataSocket->GetErrorText());
   }
 
   // Increase internal buffer size on media UDP sockets
-  SetMinBufferSize(dataSocket,    SO_RCVBUF);
-  SetMinBufferSize(dataSocket,    SO_SNDBUF);
-  SetMinBufferSize(controlSocket, SO_RCVBUF);
-  SetMinBufferSize(controlSocket, SO_SNDBUF);
+  SetMinBufferSize(*dataSocket,    SO_RCVBUF);
+  SetMinBufferSize(*dataSocket,    SO_SNDBUF);
+  SetMinBufferSize(*controlSocket, SO_RCVBUF);
+  SetMinBufferSize(*controlSocket, SO_SNDBUF);
 #endif
 
   shutdownRead = FALSE;
@@ -1420,9 +1460,8 @@ BOOL RTP_UDP::Open(PIPSocket::Address _localAddress, WORD portBase, WORD portMax
   if (canonicalName.Find('@') == P_MAX_INDEX)
     canonicalName += '@' + GetLocalHostName();
 
-  PTRACE(2, "RTP_UDP\tSession " << sessionID
-         << " created: data=" << dataSocket.GetPort()
-         << " control=" << controlSocket.GetPort()
+  PTRACE(2, "RTP_UDP\tSession " << sessionID << " created: "
+         << localAddress << ':' << localDataPort << '-' << localControlPort
          << " ssrc=" << syncSourceOut);
   return TRUE;
 }
@@ -1435,11 +1474,13 @@ void RTP_UDP::Close(BOOL reading)
       PTRACE(3, "RTP_UDP\tSession " << sessionID << ", Shutting down read.");
       syncSourceIn = 0;
       shutdownRead = TRUE;
-      PIPSocket::Address addr;
-      controlSocket.GetLocalAddress(addr);
-      if (addr == INADDR_ANY)
-        PIPSocket::GetHostAddress(addr);
-      dataSocket.WriteTo("", 1, addr, controlSocket.GetPort());
+      if (dataSocket != NULL && controlSocket != NULL) {
+        PIPSocket::Address addr;
+        controlSocket->GetLocalAddress(addr);
+        if (addr == INADDR_ANY)
+          PIPSocket::GetHostAddress(addr);
+        dataSocket->WriteTo("", 1, addr, controlSocket->GetPort());
+      }
     }
   }
   else {
@@ -1486,7 +1527,7 @@ BOOL RTP_UDP::SetRemoteSocketInfo(PIPSocket::Address address, WORD port, BOOL is
 BOOL RTP_UDP::ReadData(RTP_DataFrame & frame)
 {
   for (;;) {
-    int selectStatus = PSocket::Select(dataSocket, controlSocket, reportTimer);
+    int selectStatus = PSocket::Select(*dataSocket, *controlSocket, reportTimer);
 
     if (shutdownRead) {
       PTRACE(3, "RTP_UDP\tSession " << sessionID << ", Read shutdown.");
@@ -1518,7 +1559,7 @@ BOOL RTP_UDP::ReadData(RTP_DataFrame & frame)
         break;
 
       case 0 :
-        PTRACE(4, "RTP_UDP\tSession " << sessionID << ", Read timeout.");
+        PTRACE(5, "RTP_UDP\tSession " << sessionID << ", check for sending report.");
         if (!SendReport())
           return FALSE;
         break;
@@ -1538,18 +1579,37 @@ BOOL RTP_UDP::ReadData(RTP_DataFrame & frame)
 
 RTP_Session::SendReceiveStatus RTP_UDP::ReadDataOrControlPDU(PUDPSocket & socket,
                                                              PBYTEArray & frame,
-                                                             const char * PTRACE_name)
+                                                             BOOL fromDataChannel)
 {
+#if PTRACING
+  const char * channelName = fromDataChannel ? "Data" : "Control";
+#endif
   PIPSocket::Address addr;
   WORD port;
 
   if (socket.ReadFrom(frame.GetPointer(), frame.GetSize(), addr, port)) {
     if (ignoreOtherSources) {
-      if (remoteTransmitAddress == 0)
+      // If remote address never set from higher levels, then try and figure
+      // it out from the first packet received.
+      if (!remoteAddress.IsValid()) {
+        remoteAddress = addr;
+        PTRACE(4, "RTP\tSet remote address from first " << channelName
+               << " PDU from " << addr << ':' << port);
+      }
+      if (fromDataChannel) {
+        if (remoteDataPort == 0)
+          remoteDataPort = port;
+      }
+      else {
+        if (remoteControlPort == 0)
+          remoteControlPort = port;
+      }
+
+      if (!remoteTransmitAddress.IsValid())
         remoteTransmitAddress = addr;
       else if (remoteTransmitAddress != addr) {
         PTRACE(1, "RTP_UDP\tSession " << sessionID << ", "
-               << PTRACE_name << " PDU from incorrect host, "
+               << channelName << " PDU from incorrect host, "
                   " is " << addr << " should be " << remoteTransmitAddress);
         return RTP_Session::e_IgnorePacket;
       }
@@ -1562,11 +1622,11 @@ RTP_Session::SendReceiveStatus RTP_UDP::ReadDataOrControlPDU(PUDPSocket & socket
     case ECONNRESET :
     case ECONNREFUSED :
       PTRACE(2, "RTP_UDP\tSession " << sessionID << ", "
-             << PTRACE_name << " port on remote not ready.");
+             << channelName << " port on remote not ready.");
       return RTP_Session::e_IgnorePacket;
 
     default:
-      PTRACE(1, "RTP_UDP\t" << PTRACE_name << " read error ("
+      PTRACE(1, "RTP_UDP\t" << channelName << " read error ("
              << socket.GetErrorNumber(PChannel::LastReadError) << "): "
              << socket.GetErrorText(PChannel::LastReadError));
       return RTP_Session::e_AbortTransport;
@@ -1576,12 +1636,12 @@ RTP_Session::SendReceiveStatus RTP_UDP::ReadDataOrControlPDU(PUDPSocket & socket
 
 RTP_Session::SendReceiveStatus RTP_UDP::ReadDataPDU(RTP_DataFrame & frame)
 {
-  SendReceiveStatus status = ReadDataOrControlPDU(dataSocket, frame, "Data");
+  SendReceiveStatus status = ReadDataOrControlPDU(*dataSocket, frame, TRUE);
   if (status != e_ProcessPacket)
     return status;
 
   // Check received PDU is big enough
-  PINDEX pduSize = dataSocket.GetLastReadCount();
+  PINDEX pduSize = dataSocket->GetLastReadCount();
   if (pduSize < RTP_DataFrame::MinHeaderSize || pduSize < frame.GetHeaderSize()) {
     PTRACE(2, "RTP_UDP\tSession " << sessionID
            << ", Received data packet too small: " << pduSize << " bytes");
@@ -1597,11 +1657,11 @@ RTP_Session::SendReceiveStatus RTP_UDP::ReadControlPDU()
 {
   RTP_ControlFrame frame(2048);
 
-  SendReceiveStatus status = ReadDataOrControlPDU(controlSocket, frame, "Control");
+  SendReceiveStatus status = ReadDataOrControlPDU(*controlSocket, frame, FALSE);
   if (status != e_ProcessPacket)
     return status;
 
-  PINDEX pduSize = controlSocket.GetLastReadCount();
+  PINDEX pduSize = controlSocket->GetLastReadCount();
   if (pduSize < 4 || pduSize < 4+frame.GetPayloadSize()) {
     PTRACE(2, "RTP_UDP\tSession " << sessionID
            << ", Received control packet too small: " << pduSize << " bytes");
@@ -1621,6 +1681,10 @@ BOOL RTP_UDP::WriteData(RTP_DataFrame & frame)
     return FALSE;
   }
 
+  // Trying to send a PDU before we are set up!
+  if (!remoteAddress.IsValid() || remoteDataPort == 0)
+    return TRUE;
+
   switch (OnSendData(frame)) {
     case e_ProcessPacket :
       break;
@@ -1630,30 +1694,51 @@ BOOL RTP_UDP::WriteData(RTP_DataFrame & frame)
       return FALSE;
   }
 
-  if (dataSocket.WriteTo(frame.GetPointer(),
-                         frame.GetHeaderSize()+frame.GetPayloadSize(),
-                         remoteAddress, remoteDataPort))
-    return TRUE;
+  while (!dataSocket->WriteTo(frame.GetPointer(),
+                             frame.GetHeaderSize()+frame.GetPayloadSize(),
+                             remoteAddress, remoteDataPort)) {
+    switch (dataSocket->GetErrorNumber()) {
+      case ECONNRESET :
+      case ECONNREFUSED :
+        PTRACE(2, "RTP_UDP\tSession " << sessionID << ", data port on remote not ready.");
+        break;
 
-  PTRACE(1, "RTP_UDP\tSession " << sessionID
-         << ", Write error on data port ("
-         << dataSocket.GetErrorNumber(PChannel::LastWriteError) << "): "
-         << dataSocket.GetErrorText(PChannel::LastWriteError));
-  return FALSE;
+      default:
+        PTRACE(1, "RTP_UDP\tSession " << sessionID
+               << ", Write error on data port ("
+               << dataSocket->GetErrorNumber(PChannel::LastWriteError) << "): "
+               << dataSocket->GetErrorText(PChannel::LastWriteError));
+        return FALSE;
+    }
+  }
+
+  return TRUE;
 }
 
 
 BOOL RTP_UDP::WriteControl(RTP_ControlFrame & frame)
 {
-  if (controlSocket.WriteTo(frame.GetPointer(), frame.GetCompoundSize(),
-                            remoteAddress, remoteControlPort))
+  // Trying to send a PDU before we are set up!
+  if (!remoteAddress.IsValid() || remoteControlPort == 0)
     return TRUE;
 
-  PTRACE(1, "RTP_UDP\tSession " << sessionID
-         << ", Write error on control port ("
-         << controlSocket.GetErrorNumber(PChannel::LastWriteError) << "): "
-         << controlSocket.GetErrorText(PChannel::LastWriteError));
-  return FALSE;
+  while (!controlSocket->WriteTo(frame.GetPointer(), frame.GetCompoundSize(),
+                                remoteAddress, remoteControlPort)) {
+    switch (controlSocket->GetErrorNumber()) {
+      case ECONNRESET :
+      case ECONNREFUSED :
+        PTRACE(2, "RTP_UDP\tSession " << sessionID << ", control port on remote not ready.");
+        break;
+
+      default:
+        PTRACE(1, "RTP_UDP\tSession " << sessionID
+               << ", Write error on control port ("
+               << controlSocket->GetErrorNumber(PChannel::LastWriteError) << "): "
+               << controlSocket->GetErrorText(PChannel::LastWriteError));
+    }
+  }
+
+  return TRUE;
 }
 
 
