@@ -77,7 +77,7 @@ void *nb_encoder_init(SpeexMode *m)
    mode=(SpeexNBMode *)m->mode;
    st = (EncState*)speex_alloc(sizeof(EncState));
    st->mode=m;
-   /* Codec parameters, should eventually have several "modes"*/
+
    st->frameSize = mode->frameSize;
    st->windowSize = st->frameSize*3/2;
    st->nbSubframes=mode->frameSize/mode->subframeSize;
@@ -93,10 +93,10 @@ void *nb_encoder_init(SpeexMode *m)
    st->preemph = mode->preemph;
   
    st->submodes=mode->submodes;
-   st->submodeID=mode->defaultSubmode;
+   st->submodeID=st->submodeSelect=mode->defaultSubmode;
    st->pre_mem=0;
    st->pre_mem2=0;
-   st->bounded_pitch = 0;
+   st->bounded_pitch = 1;
 
    /* Allocating input buffer */
    st->inBuf = (float*)speex_alloc(st->bufSize*sizeof(float));
@@ -130,7 +130,7 @@ void *nb_encoder_init(SpeexMode *m)
 
    st->autocorr = (float*)speex_alloc((st->lpcSize+1)*sizeof(float));
 
-   st->stack = (float*)speex_alloc(20000*sizeof(float));
+   st->stack = (float*)speex_alloc(4000*sizeof(float));
 
    st->buf2 = (float*)speex_alloc(st->windowSize*sizeof(float));
 
@@ -162,16 +162,18 @@ void *nb_encoder_init(SpeexMode *m)
 
    st->pitch = (int*)speex_alloc(st->nbSubframes*sizeof(int));
 
-   if (1) {
-      st->vbr = (VBRState*)speex_alloc(sizeof(VBRState));
-      vbr_init(st->vbr);
-      st->vbr_quality = 8;
-      st->vbr_enabled = 0;
-   } else {
-      st->vbr = 0;
-   }
+   st->vbr = (VBRState*)speex_alloc(sizeof(VBRState));
+   vbr_init(st->vbr);
+   st->vbr_quality = 8;
+   st->vbr_enabled = 0;
+   st->vad_enabled = 0;
+   st->dtx_enabled = 0;
+   st->abr_enabled = 0;
+   st->abr_drift = 0;
+
    st->complexity=2;
    st->sampling_rate=8000;
+   st->dtx_count=0;
 
    return st;
 }
@@ -219,17 +221,18 @@ void nb_encoder_destroy(void *state)
    speex_free((float*)st);
 }
 
-void nb_encode(void *state, float *in, SpeexBits *bits)
+int nb_encode(void *state, float *in, SpeexBits *bits)
 {
    EncState *st;
    int i, sub, roots;
    int ol_pitch;
    float ol_pitch_coef;
    float ol_gain;
-   float delta_qual=0;
    float *res, *target, *mem;
    void *stack;
    float *syn_resp;
+   float lsp_dist=0;
+   float *orig;
 
    st=(EncState *)state;
    stack=st->stack;
@@ -291,24 +294,20 @@ void nb_encode(void *state, float *in, SpeexBits *bits)
       }
    }
 
-   /* LSP Quantization */
-   if (st->first)
-   {
-      for (i=0;i<st->lpcSize;i++)
-         st->old_lsp[i] = st->lsp[i];
-   }
 
-   if (0) {
-      float dd=0;
-      for (i=0;i<st->lpcSize;i++)
-         dd += fabs(st->old_lsp[i] - st->lsp[i]);
-      printf ("lspdist = %f\n", dd);
-   }
+   lsp_dist=0;
+   for (i=0;i<st->lpcSize;i++)
+      lsp_dist += (st->old_lsp[i] - st->lsp[i])*(st->old_lsp[i] - st->lsp[i]);
+   /*printf ("%f\n", lsp_dist);*/
 
    /* Whole frame analysis (open-loop estimation of pitch and excitation gain) */
    {
-      for (i=0;i<st->lpcSize;i++)
-         st->interp_lsp[i] = .5*st->old_lsp[i] + .5*st->lsp[i];
+      if (st->first)
+         for (i=0;i<st->lpcSize;i++)
+            st->interp_lsp[i] = st->lsp[i];
+      else
+         for (i=0;i<st->lpcSize;i++)
+            st->interp_lsp[i] = .375*st->old_lsp[i] + .625*st->lsp[i];
 
       lsp_enforce_margin(st->interp_lsp, st->lpcSize, .002);
 
@@ -319,11 +318,11 @@ void nb_encode(void *state, float *in, SpeexBits *bits)
 
 
       /*Open-loop pitch*/
-      if (!st->submodes[st->submodeID] || st->vbr_enabled || SUBMODE(forced_pitch_gain) ||
+      if (!st->submodes[st->submodeID] || st->vbr_enabled || st->vad_enabled || SUBMODE(forced_pitch_gain) ||
           SUBMODE(lbr_pitch) != -1)
       {
-         int nol_pitch[4];
-         float nol_pitch_coef[4];
+         int nol_pitch[6];
+         float nol_pitch_coef[6];
          
          bw_lpc(st->gamma1, st->interp_lpc, st->bw_lpc1, st->lpcSize);
          bw_lpc(st->gamma2, st->interp_lpc, st->bw_lpc2, st->lpcSize);
@@ -331,20 +330,24 @@ void nb_encode(void *state, float *in, SpeexBits *bits)
          filter_mem2(st->frame, st->bw_lpc1, st->bw_lpc2, st->sw, st->frameSize, st->lpcSize, st->mem_sw_whole);
 
          open_loop_nbest_pitch(st->sw, st->min_pitch, st->max_pitch, st->frameSize, 
-                               nol_pitch, nol_pitch_coef, 4, stack);
+                               nol_pitch, nol_pitch_coef, 6, stack);
          ol_pitch=nol_pitch[0];
          ol_pitch_coef = nol_pitch_coef[0];
+         /*printf ("%f %d %d %d %d %d %d ", ol_pitch_coef, nol_pitch[0], nol_pitch[1], nol_pitch[2], nol_pitch[3], nol_pitch[4], nol_pitch[5]);*/
          /*Try to remove pitch multiples*/
-         for (i=1;i<4;i++)
+         for (i=1;i<6;i++)
          {
-            if ((nol_pitch_coef[i] > .85*ol_pitch_coef) && 
-                (fabs(2*nol_pitch[i]-ol_pitch)<=2 || fabs(3*nol_pitch[i]-ol_pitch)<=4 || 
-                 fabs(4*nol_pitch[i]-ol_pitch)<=6 || fabs(5*nol_pitch[i]-ol_pitch)<=8))
+            if ((nol_pitch_coef[i]>.85*ol_pitch_coef) && 
+                (fabs(nol_pitch[i]-ol_pitch/2.0)<=1 || fabs(nol_pitch[i]-ol_pitch/3.0)<=1 || 
+                 fabs(nol_pitch[i]-ol_pitch/4.0)<=1 || fabs(nol_pitch[i]-ol_pitch/5.0)<=1))
             {
                /*ol_pitch_coef=nol_pitch_coef[i];*/
                ol_pitch = nol_pitch[i];
             }
          }
+         /*printf ("%d\n", ol_pitch);*/
+         /*if (ol_pitch>50)
+           ol_pitch/=2;*/
          /*ol_pitch_coef = sqrt(ol_pitch_coef);*/
          /*printf ("ol_pitch: %d %f\n", ol_pitch, ol_pitch_coef);*/
       } else {
@@ -358,29 +361,107 @@ void nb_encode(void *state, float *in, SpeexBits *bits)
       ol_gain=0;
       for (i=0;i<st->frameSize;i++)
          ol_gain += st->exc[i]*st->exc[i];
+      /*for (i=0;i<160;i++)
+        printf ("%f\n", st->exc[i]);*/
       
       ol_gain=sqrt(1+ol_gain/st->frameSize);
    }
 
-   /*Experimental VBR stuff*/
-   if (st->vbr)
+   /*VBR stuff*/
+   if (st->vbr && (st->vbr_enabled||st->vad_enabled))
    {
-      delta_qual = vbr_analysis(st->vbr, in, st->frameSize, ol_pitch, ol_pitch_coef);
+      
+      if (st->abr_enabled)
+      {
+         float qual_change=0;
+         if (st->abr_drift2 * st->abr_drift > 0)
+         {
+            /* Only adapt if long-term and short-term drift are the same sign */
+            qual_change = -.00001*st->abr_drift/(1+st->abr_count);
+            if (qual_change>.05)
+               qual_change=.05;
+            if (qual_change<-.05)
+               qual_change=-.05;
+         }
+         st->vbr_quality += qual_change;
+         if (st->vbr_quality>10)
+            st->vbr_quality=10;
+         if (st->vbr_quality<0)
+            st->vbr_quality=0;
+         /*printf ("%f %f\n", st->abr_drift/(.1+st->abr_count), st->vbr_quality);*/
+      }
+
+      st->relative_quality = vbr_analysis(st->vbr, in, st->frameSize, ol_pitch, ol_pitch_coef);
       /*if (delta_qual<0)*/
-         delta_qual*=.1*(3+st->vbr_quality);
+      /*  delta_qual*=.1*(3+st->vbr_quality);*/
       if (st->vbr_enabled) 
       {
-         int qual = (int)floor(st->vbr_quality+delta_qual+.5);
-         if (qual<1 && delta_qual>-3.5)
-            qual=1;
-         if (qual<0)
-            qual=0;
-         if (qual>10)
-            qual=10;
-         if (qual==10 && st->vbr_quality<10)
-            qual=9;
-         speex_encoder_ctl(state, SPEEX_SET_QUALITY, &qual);
-      }
+         int mode;
+         mode = 7;
+         while (mode)
+         {
+            int v1;
+            float thresh;
+            v1=(int)floor(st->vbr_quality);
+            if (v1==10)
+               thresh = vbr_nb_thresh[mode][v1];
+            else
+               thresh = (st->vbr_quality-v1)*vbr_nb_thresh[mode][v1+1] + (1+v1-st->vbr_quality)*vbr_nb_thresh[mode][v1];
+            if (st->relative_quality > thresh)
+               break;
+            mode--;
+         }
+         
+         if (mode==0)
+         {
+            if (st->submodeID>1 || lsp_dist>.05 || st->dtx_count>20)
+            {
+               mode=1;
+               st->dtx_count=0;
+            } else {
+               mode=0;
+               st->dtx_count++;
+            }
+         } else {
+            if (st->submodeID>1)
+               st->dtx_count=0;
+         }
+
+         /*fprintf(stderr, "");
+         fprintf (stderr, "encode %f %d\n", st->relative_quality, mode);
+         fprintf(stderr, "encode: %d %d\n",st->submodeID, mode);*/
+
+         speex_encoder_ctl(state, SPEEX_SET_MODE, &mode);
+
+         if (st->abr_enabled)
+         {
+            int bitrate;
+            speex_encoder_ctl(state, SPEEX_GET_BITRATE, &bitrate);
+            st->abr_drift+=(bitrate-st->abr_enabled);
+            st->abr_drift2 = .95*st->abr_drift2 + .05*(bitrate-st->abr_enabled);
+            st->abr_count += 1.0;
+         }
+         /*fprintf(stderr, "encode: %d %d\n",st->submodeID, mode);*/
+      } else {
+         /*VAD only case*/
+         int mode;
+         if (st->relative_quality<2)
+         {
+            if (st->submodeID>1 || lsp_dist>.05 || !st->dtx_enabled || st->dtx_count>20)
+            {
+               st->dtx_count=0;
+               mode=1;
+            } else {
+               mode=0;
+               st->dtx_count++;
+            }
+         } else
+            mode=st->submodeSelect;
+         /*speex_encoder_ctl(state, SPEEX_SET_MODE, &mode);*/
+         st->submodeID=mode;
+      } 
+   } else {
+      st->relative_quality = -1;
    }
    /*printf ("VBR quality = %f\n", vbr_qual);*/
 
@@ -400,19 +481,28 @@ void nb_encode(void *state, float *in, SpeexBits *bits)
       for (i=0;i<st->lpcSize;i++)
          st->mem_sw[i]=0;
       st->first=1;
+      st->bounded_pitch = 1;
 
       /* Final signal synthesis from excitation */
-      iir_mem2(st->exc, st->interp_qlpc, st->frame, st->subframeSize, st->lpcSize, st->mem_sp);
+      iir_mem2(st->exc, st->interp_qlpc, st->frame, st->frameSize, st->lpcSize, st->mem_sp);
 
       in[0] = st->frame[0] + st->preemph*st->pre_mem2;
       for (i=1;i<st->frameSize;i++)
          in[i]=st->frame[i] + st->preemph*in[i-1];
       st->pre_mem2=in[st->frameSize-1];
 
-      return;
+      return 0;
 
    }
 
+   /* LSP Quantization */
+   if (st->first)
+   {
+      for (i=0;i<st->lpcSize;i++)
+         st->old_lsp[i] = st->lsp[i];
+   }
+
+
    /*Quantize LSPs*/
 #if 1 /*0 for unquantized*/
    SUBMODE(lsp_quant)(st->lsp, st->qlsp, st->lpcSize, bits);
@@ -464,6 +554,9 @@ void nb_encode(void *state, float *in, SpeexBits *bits)
    target = PUSH(stack, st->subframeSize, float);
    syn_resp = PUSH(stack, st->subframeSize, float);
    mem = PUSH(stack, st->lpcSize, float);
+   orig = PUSH(stack, st->frameSize, float);
+   for (i=0;i<st->frameSize;i++)
+      orig[i]=st->frame[i];
 
    /* Loop on sub-frames */
    for (sub=0;sub<st->nbSubframes;sub++)
@@ -513,7 +606,6 @@ void nb_encode(void *state, float *in, SpeexBits *bits)
          st->pi_gain[sub] += tmp*st->interp_qlpc[i];
          tmp = -tmp;
       }
-     
 
       /* Compute bandwidth-expanded (unquantized) LPCs for perceptual weighting */
       bw_lpc(st->gamma1, st->interp_lpc, st->bw_lpc1, st->lpcSize);
@@ -620,7 +712,9 @@ void nb_encode(void *state, float *in, SpeexBits *bits)
          for (i=0;i<st->subframeSize;i++)
             ener+=st->buf2[i]*st->buf2[i];
          ener=sqrt(.1+ener/st->subframeSize);
-
+         /*for (i=0;i<st->subframeSize;i++)
+            printf ("%f\n", st->buf2[i]/ener);
+         */
          
          ener /= ol_gain;
 
@@ -646,6 +740,8 @@ void nb_encode(void *state, float *in, SpeexBits *bits)
 
          ener*=ol_gain;
 
+         /*printf ("%f %f\n", ener, ol_gain);*/
+
          ener_1 = 1/ener;
 
          if (0) {
@@ -712,24 +808,41 @@ void nb_encode(void *state, float *in, SpeexBits *bits)
    }
 
    /* Store the LSPs for interpolation in the next frame */
-   for (i=0;i<st->lpcSize;i++)
-      st->old_lsp[i] = st->lsp[i];
-   for (i=0;i<st->lpcSize;i++)
-      st->old_qlsp[i] = st->qlsp[i];
+   if (st->submodeID>=1)
+   {
+      for (i=0;i<st->lpcSize;i++)
+         st->old_lsp[i] = st->lsp[i];
+      for (i=0;i<st->lpcSize;i++)
+         st->old_qlsp[i] = st->qlsp[i];
+   }
 
    /* The next frame will not be the first (Duh!) */
    st->first = 0;
 
+   {
+      float ener=0, err=0;
+      float snr;
+      for (i=0;i<st->frameSize;i++)
+      {
+         ener+=st->frame[i]*st->frame[i];
+         err += (st->frame[i]-orig[i])*(st->frame[i]-orig[i]);
+      }
+      snr = 10*log10((ener+1)/(err+1));
+      /*printf ("%f %f %f\n", snr, ener, err);*/
+   }
+
    /* Replace input by synthesized speech */
    in[0] = st->frame[0] + st->preemph*st->pre_mem2;
    for (i=1;i<st->frameSize;i++)
      in[i]=st->frame[i] + st->preemph*in[i-1];
    st->pre_mem2=in[st->frameSize-1];
 
-   if (SUBMODE(innovation_quant) == noise_codebook_quant)
+   if (SUBMODE(innovation_quant) == noise_codebook_quant || st->submodeID==0)
       st->bounded_pitch = 1;
    else
       st->bounded_pitch = 0;
+
+   return 1;
 }
 
 
@@ -763,7 +876,7 @@ void *nb_decoder_init(SpeexMode *m)
    st->pre_mem=0;
    st->lpc_enh_enabled=0;
 
-   st->stack = speex_alloc(20000*sizeof(float));
+   st->stack = speex_alloc(2000*sizeof(float));
 
    st->inBuf = (float*)speex_alloc(st->bufSize*sizeof(float));
    st->frame = st->inBuf + st->bufSize - st->windowSize;
@@ -780,17 +893,26 @@ void *nb_decoder_init(SpeexMode *m)
    st->old_qlsp = (float*)speex_alloc(st->lpcSize*sizeof(float));
    st->interp_qlsp = (float*)speex_alloc(st->lpcSize*sizeof(float));
    st->mem_sp = (float*)speex_alloc(5*st->lpcSize*sizeof(float));
+   st->comb_mem = (CombFilterMem *) speex_alloc(sizeof(CombFilterMem));
+   comp_filter_mem_init (st->comb_mem);
 
    st->pi_gain = (float*)speex_alloc(st->nbSubframes*sizeof(float));
    st->last_pitch = 40;
    st->count_lost=0;
+   st->pitch_gain_buf[0] = st->pitch_gain_buf[1] = st->pitch_gain_buf[2] = 0;
+   st->pitch_gain_buf_idx = 0;
+
    st->sampling_rate=8000;
+   st->last_ol_gain = 0;
 
    st->user_callback.func = &speex_default_user_handler;
    st->user_callback.data = NULL;
    for (i=0;i<16;i++)
       st->speex_callbacks[i].func = NULL;
 
+   st->voc_m1=st->voc_m2=st->voc_mean=0;
+   st->voc_offset=0;
+
    return st;
 }
 
@@ -808,15 +930,34 @@ void nb_decoder_destroy(void *state)
    speex_free(st->stack);
    speex_free(st->mem_sp);
    speex_free(st->pi_gain);
+   speex_free(st->comb_mem);
    
    speex_free(state);
 }
 
+#define median3(a, b, c)	((a) < (b) ? ((b) < (c) ? (b) : ((a) < (c) ? (c) : (a))) : ((c) < (b) ? (b) : ((c) < (a) ? (c) : (a))))
+
 static void nb_decode_lost(DecState *st, float *out, void *stack)
 {
    int i, sub;
    float *awk1, *awk2, *awk3;
-   /*float exc_ener=0,g;*/
+   float pitch_gain, fact, gain_med;
+
+   fact = exp(-.04*st->count_lost*st->count_lost);
+   gain_med = median3(st->pitch_gain_buf[0], st->pitch_gain_buf[1], st->pitch_gain_buf[2]);
+   if (gain_med < st->last_pitch_gain)
+      st->last_pitch_gain = gain_med;
+   
+   pitch_gain = st->last_pitch_gain;
+   if (pitch_gain>.95)
+      pitch_gain=.95;
+
+   pitch_gain *= fact;
+   
+#ifdef DEBUG
+   printf ("recovered unquantized pitch: %d, gain: %f (fact=%f, buf=[%f %f %f], med=%f)\n", st->last_pitch, pitch_gain, fact, st->pitch_gain_buf[0], st->pitch_gain_buf[1], st->pitch_gain_buf[2], gain_med);
+#endif
+
    /* Shift all buffers by one frame */
    speex_move(st->inBuf, st->inBuf+st->frameSize, (st->bufSize-st->frameSize)*sizeof(float));
    speex_move(st->excBuf, st->excBuf+st->frameSize, (st->bufSize-st->frameSize)*sizeof(float));
@@ -854,17 +995,31 @@ static void nb_decode_lost(DecState *st, float *out, void *stack)
          bw_lpc(k1, st->interp_qlpc, awk1, st->lpcSize);
          bw_lpc(k2, st->interp_qlpc, awk2, st->lpcSize);
          bw_lpc(k3, st->interp_qlpc, awk3, st->lpcSize);
-         
       }
         
       /* Make up a plausible excitation */
       /* THIS CAN BE IMPROVED */
+      /*if (pitch_gain>.95)
+        pitch_gain=.95;*/
+      /*printf ("%f\n", pitch_gain);*/
+      {
+         float innov_gain=0;
+         for (i=0;i<st->frameSize;i++)
+            innov_gain += st->innov[i]*st->innov[i];
+         innov_gain=sqrt(innov_gain/160);
       for (i=0;i<st->subframeSize;i++)
       {
-         exc[i]=st->last_pitch_gain*exc[i-st->last_pitch] + 
-         .8*st->innov[i+offset];
+#if 0
+         exc[i] = pitch_gain * exc[i - st->last_pitch] + fact*sqrt(1-pitch_gain)*st->innov[i+offset];
+         /*Just so it give the same lost packets as with if 0*/
+         rand();
+#else
+         /*exc[i]=pitch_gain*exc[i-st->last_pitch] +  fact*st->innov[i+offset];*/
+         exc[i]=pitch_gain*exc[i-st->last_pitch] + 
+         fact*sqrt(1-pitch_gain)*3*innov_gain*((((float)rand())/RAND_MAX)-.5);
+#endif
+      }
       }
-
       for (i=0;i<st->subframeSize;i++)
          sp[i]=exc[i];
       
@@ -880,8 +1035,7 @@ static void nb_decode_lost(DecState *st, float *out, void *stack)
             st->mem_sp[st->lpcSize+i] = 0;
          iir_mem2(sp, st->interp_qlpc, sp, st->subframeSize, st->lpcSize, 
                      st->mem_sp);
-      }
-  
+      }      
    }
 
    out[0] = st->frame[0] + st->preemph*st->pre_mem;
@@ -891,76 +1045,87 @@ static void nb_decode_lost(DecState *st, float *out, void *stack)
    
    st->first = 0;
    st->count_lost++;
+   st->pitch_gain_buf[st->pitch_gain_buf_idx++] = pitch_gain;
+   if (st->pitch_gain_buf_idx > 2) /* rollover */
+      st->pitch_gain_buf_idx = 0;
 }
 
-
 int nb_decode(void *state, SpeexBits *bits, float *out)
 {
    DecState *st;
    int i, sub;
    int pitch;
    float pitch_gain[3];
-   float ol_gain;
+   float ol_gain=0;
    int ol_pitch=0;
    float ol_pitch_coef=0;
    int best_pitch=40;
-   float best_pitch_gain=-1;
+   float best_pitch_gain=0;
    int wideband;
    int m;
    void *stack;
    float *awk1, *awk2, *awk3;
+   float pitch_average=0;
+
    st=(DecState*)state;
    stack=st->stack;
 
-   /* If bits is NULL, consider the packet to be lost (what could we do anyway) */
-   if (!bits)
+   /* Check if we're in DTX mode*/
+   if (!bits && st->submodeID<2)
    {
-      nb_decode_lost(st, out, stack);
-      return 0;
-   }
-
-   /* Search for next narrwoband block (handle requests, skip wideband blocks) */
-   do {
-      wideband = speex_bits_unpack_unsigned(bits, 1);
-      if (wideband) /* Skip wideband block (for compatibility) */
+      st->submodeID=0;
+   } else 
+   {
+      /* If bits is NULL, consider the packet to be lost (what could we do anyway) */
+      if (!bits)
       {
-         int submode;
-         int advance;
-         submode = speex_bits_unpack_unsigned(bits, SB_SUBMODE_BITS);
-         advance = submode;
-         speex_mode_query(&speex_wb_mode, SPEEX_SUBMODE_BITS_PER_FRAME, &advance);
-         advance -= (SB_SUBMODE_BITS+1);
-         speex_bits_advance(bits, advance);
+         nb_decode_lost(st, out, stack);
+         return 0;
+      }
+
+      /* Search for next narrwoband block (handle requests, skip wideband blocks) */
+      do {
          wideband = speex_bits_unpack_unsigned(bits, 1);
-         if (wideband)
+         if (wideband) /* Skip wideband block (for compatibility) */
          {
-            fprintf (stderr, "Corrupted stream?\n");
+            int submode;
+            int advance;
+            advance = submode = speex_bits_unpack_unsigned(bits, SB_SUBMODE_BITS);
+            speex_mode_query(&speex_wb_mode, SPEEX_SUBMODE_BITS_PER_FRAME, &advance);
+            advance -= (SB_SUBMODE_BITS+1);
+            speex_bits_advance(bits, advance);
+            wideband = speex_bits_unpack_unsigned(bits, 1);
+            if (wideband)
+            {
+               fprintf (stderr, "Corrupted stream?\n");
+            }
          }
-      }
 
-      m = speex_bits_unpack_unsigned(bits, 4);
-      if (m==15) /* We found a terminator */
-      {
-         return -1;
-      } else if (m==14) /* Speex in-band request */
-      {
-         int ret = speex_inband_handler(bits, st->speex_callbacks, state);
-         if (ret)
-            return ret;
-      } else if (m==13) /* User in-band request */
-      {
-         int ret = st->user_callback.func(bits, state, st->user_callback.data);
-         if (ret)
-            return ret;
-      } else if (m>7) /* Invalid mode */
-      {
-         return -2;
-      }
+         m = speex_bits_unpack_unsigned(bits, 4);
+         if (m==15) /* We found a terminator */
+         {
+            return -1;
+         } else if (m==14) /* Speex in-band request */
+         {
+            int ret = speex_inband_handler(bits, st->speex_callbacks, state);
+            if (ret)
+               return ret;
+         } else if (m==13) /* User in-band request */
+         {
+            int ret = st->user_callback.func(bits, state, st->user_callback.data);
+            if (ret)
+               return ret;
+         } else if (m>7) /* Invalid mode */
+         {
+            return -2;
+         }
       
-   } while (m>7);
+      } while (m>7);
 
-   /* Get the sub-mode that was used */
-   st->submodeID = m;
+      /* Get the sub-mode that was used */
+      st->submodeID = m;
+
+   }
 
    /* Shift all buffers by one frame */
    speex_move(st->inBuf, st->inBuf+st->frameSize, (st->bufSize-st->frameSize)*sizeof(float));
@@ -969,12 +1134,30 @@ int nb_decode(void *state, SpeexBits *bits, float *out)
    /* If null mode (no transmission), just set a couple things to zero*/
    if (st->submodes[st->submodeID] == NULL)
    {
-      for (i=0;i<st->frameSize;i++)
-         st->exc[i]=0;
+      float *lpc;
+      lpc = PUSH(stack,11, float);
+      bw_lpc(.93, st->interp_qlpc, lpc, 10);
+      /*for (i=0;i<st->frameSize;i++)
+        st->exc[i]=0;*/
+      {
+         float innov_gain=0;
+         float pitch_gain=st->last_pitch_gain;
+         if (pitch_gain>.6)
+            pitch_gain=.6;
+         for (i=0;i<st->frameSize;i++)
+            innov_gain += st->innov[i]*st->innov[i];
+         innov_gain=sqrt(innov_gain/160);
+         for (i=0;i<st->frameSize;i++)
+         {
+            st->exc[i]=3*innov_gain*((((float)rand())/RAND_MAX)-.5);
+         }
+      }
+
+
       st->first=1;
-      
+
       /* Final signal synthesis from excitation */
-      iir_mem2(st->exc, st->interp_qlpc, st->frame, st->subframeSize, st->lpcSize, st->mem_sp);
+      iir_mem2(st->exc, lpc, st->frame, st->frameSize, st->lpcSize, st->mem_sp);
 
       out[0] = st->frame[0] + st->preemph*st->pre_mem;
       for (i=1;i<st->frameSize;i++)
@@ -987,6 +1170,18 @@ int nb_decode(void *state, SpeexBits *bits, float *out)
    /* Unquantize LSPs */
    SUBMODE(lsp_unquant)(st->qlsp, st->lpcSize, bits);
 
+   /*Damp memory if a frame was lost and the LSP changed too much*/
+   if (st->count_lost)
+   {
+      float lsp_dist=0, fact;
+      for (i=0;i<st->lpcSize;i++)
+         lsp_dist += fabs(st->old_qlsp[i] - st->qlsp[i]);
+      fact = .6*exp(-.2*lsp_dist);
+      for (i=0;i<2*st->lpcSize;i++)
+         st->mem_sp[i] *= fact;
+   }
+
+
    /* Handle first frame and lost-packet case */
    if (st->first || st->count_lost)
    {
@@ -1092,14 +1287,22 @@ int nb_decode(void *state, SpeexBits *bits, float *out)
             margin = SUBMODE(lbr_pitch);
             if (margin)
             {
+/* GT - need optimization?
                if (ol_pitch < st->min_pitch+margin-1)
                   ol_pitch=st->min_pitch+margin-1;
                if (ol_pitch > st->max_pitch-margin)
                   ol_pitch=st->max_pitch-margin;
                pit_min = ol_pitch-margin+1;
                pit_max = ol_pitch+margin;
+*/
+               pit_min = ol_pitch-margin+1;
+               if (pit_min < st->min_pitch)
+		  pit_min = st->min_pitch;
+               pit_max = ol_pitch+margin;
+               if (pit_max > st->max_pitch)
+		  pit_max = st->max_pitch;
             } else {
-               pit_min=pit_max=ol_pitch;
+               pit_min = pit_max = ol_pitch;
             }
          } else {
             pit_min = st->min_pitch;
@@ -1108,15 +1311,36 @@ int nb_decode(void *state, SpeexBits *bits, float *out)
 
          /* Pitch synthesis */
          SUBMODE(ltp_unquant)(exc, pit_min, pit_max, ol_pitch_coef, SUBMODE(ltp_params), 
-                              st->subframeSize, &pitch, &pitch_gain[0], bits, stack, st->count_lost);
+                              st->subframeSize, &pitch, &pitch_gain[0], bits, stack, st->count_lost, offset, st->last_pitch_gain);
          
-         tmp = (pitch_gain[0]+pitch_gain[1]+pitch_gain[2]);
+         /* If we had lost frames, check energy of last received frame */
+         if (st->count_lost && ol_gain < st->last_ol_gain)
+         {
+            float fact = ol_gain/(st->last_ol_gain+1);
+            for (i=0;i<st->subframeSize;i++)
+               exc[i]*=fact;
+         }
+
+         tmp = fabs(pitch_gain[0]+pitch_gain[1]+pitch_gain[2]);
+         tmp = fabs(pitch_gain[1]);
+         if (pitch_gain[0]>0)
+            tmp += pitch_gain[0];
+         else
+            tmp -= .5*pitch_gain[0];
+         if (pitch_gain[2]>0)
+            tmp += pitch_gain[2];
+         else
+            tmp -= .5*pitch_gain[0];
+
+
+         pitch_average += tmp;
          if (tmp>best_pitch_gain)
          {
             best_pitch = pitch;
-            best_pitch_gain = tmp*.9;
-            if (best_pitch_gain>.85)
-               best_pitch_gain=.85;
+	    best_pitch_gain = tmp;
+	    /*            best_pitch_gain = tmp*.9;
+	                if (best_pitch_gain>.85)
+                        best_pitch_gain=.85;*/
          }
       } else {
          fprintf (stderr, "No pitch prediction, what's wrong\n");
@@ -1158,9 +1382,42 @@ int nb_decode(void *state, SpeexBits *bits, float *out)
          /* De-normalize innovation and update excitation */
          for (i=0;i<st->subframeSize;i++)
             innov[i]*=ener;
-         for (i=0;i<st->subframeSize;i++)
-            exc[i]+=innov[i];
 
+         /*Vocoder mode*/
+         if (st->submodeID==1) 
+         {
+            float g=ol_pitch_coef;
+
+            
+            for (i=0;i<st->subframeSize;i++)
+               exc[i]=0;
+            while (st->voc_offset<st->subframeSize)
+            {
+               if (st->voc_offset>=0)
+                  exc[st->voc_offset]=sqrt(1.0*ol_pitch);
+               st->voc_offset+=ol_pitch;
+            }
+            st->voc_offset -= st->subframeSize;
+
+            g=.5+2*(g-.6);
+            if (g<0)
+               g=0;
+            if (g>1)
+               g=1;
+            for (i=0;i<st->subframeSize;i++)
+            {
+               int tmp=exc[i];
+               exc[i]=.8*g*exc[i]*ol_gain + .6*g*st->voc_m1*ol_gain + .5*g*innov[i] - .5*g*st->voc_m2 + (1-g)*innov[i];
+               st->voc_m1 = tmp;
+               st->voc_m2=innov[i];
+               st->voc_mean = .95*st->voc_mean + .05*exc[i];
+               exc[i]-=st->voc_mean;
+            }
+         } else {
+            for (i=0;i<st->subframeSize;i++)
+               exc[i]+=innov[i];
+         }
+         /*printf ("%f %f\n", ener, ol_gain);*/
          /* Decode second codebook (only for some modes) */
          if (SUBMODE(double_codebook))
          {
@@ -1181,9 +1438,9 @@ int nb_decode(void *state, SpeexBits *bits, float *out)
          sp[i]=exc[i];
 
       /* Signal synthesis */
-      if (st->lpc_enh_enabled && SUBMODE(comb_gain>0))
+      if (st->lpc_enh_enabled && SUBMODE(comb_gain)>0)
          comb_filter(exc, sp, st->interp_qlpc, st->lpcSize, st->subframeSize,
-                              pitch, pitch_gain, .5);
+                              pitch, pitch_gain, SUBMODE(comb_gain), st->comb_mem);
       if (st->lpc_enh_enabled)
       {
          /* Use enhanced LPC filter */
@@ -1215,7 +1472,17 @@ int nb_decode(void *state, SpeexBits *bits, float *out)
    st->first = 0;
    st->count_lost=0;
    st->last_pitch = best_pitch;
-   st->last_pitch_gain = best_pitch_gain;
+   st->last_pitch_gain = .25*pitch_average;
+   st->pitch_gain_buf[st->pitch_gain_buf_idx++] = st->last_pitch_gain;
+   if (st->pitch_gain_buf_idx > 2) /* rollover */
+      st->pitch_gain_buf_idx = 0;
+
+   st->last_ol_gain = ol_gain;
+
+
+#ifdef DEBUG
+printf ("last pitch: {%d, %f} buf=[%f %f %f]\n", st->last_pitch, st->last_pitch_gain, st->pitch_gain_buf[0], st->pitch_gain_buf[1], st->pitch_gain_buf[2]);
+#endif
 
    return 0;
 }
@@ -1231,7 +1498,7 @@ void nb_encoder_ctl(void *state, int request, void *ptr)
       break;
    case SPEEX_SET_LOW_MODE:
    case SPEEX_SET_MODE:
-      st->submodeID = (*(int*)ptr);
+      st->submodeSelect = st->submodeID = (*(int*)ptr);
       break;
    case SPEEX_GET_LOW_MODE:
    case SPEEX_GET_MODE:
@@ -1243,6 +1510,46 @@ void nb_encoder_ctl(void *state, int request, void *ptr)
    case SPEEX_GET_VBR:
       (*(int*)ptr) = st->vbr_enabled;
       break;
+   case SPEEX_SET_VAD:
+      st->vad_enabled = (*(int*)ptr);
+      break;
+   case SPEEX_GET_VAD:
+      (*(int*)ptr) = st->vad_enabled;
+      break;
+   case SPEEX_SET_DTX:
+      st->dtx_enabled = (*(int*)ptr);
+      break;
+   case SPEEX_GET_DTX:
+      (*(int*)ptr) = st->dtx_enabled;
+      break;
+   case SPEEX_SET_ABR:
+      st->abr_enabled = (*(int*)ptr);
+      st->vbr_enabled = 1;
+      {
+         int i=10, rate, target;
+         float vbr_qual;
+         target = (*(int*)ptr);
+         while (i>=0)
+         {
+            speex_encoder_ctl(st, SPEEX_SET_QUALITY, &i);
+            speex_encoder_ctl(st, SPEEX_GET_BITRATE, &rate);
+            if (rate <= target)
+               break;
+            i--;
+         }
+         vbr_qual=i;
+         if (vbr_qual<0)
+            vbr_qual=0;
+         speex_encoder_ctl(st, SPEEX_SET_VBR_QUALITY, &vbr_qual);
+         st->abr_count=0;
+         st->abr_drift=0;
+         st->abr_drift2=0;
+      }
+      
+      break;
+   case SPEEX_GET_ABR:
+      (*(int*)ptr) = st->abr_enabled;
+      break;
    case SPEEX_SET_VBR_QUALITY:
       st->vbr_quality = (*(float*)ptr);
       break;
@@ -1252,30 +1559,11 @@ void nb_encoder_ctl(void *state, int request, void *ptr)
    case SPEEX_SET_QUALITY:
       {
          int quality = (*(int*)ptr);
-         /*
-         if (quality<=0)
-            st->submodeID = 0;
-         else if (quality<=1)
-            st->submodeID = 1;
-         else if (quality<=2)
-            st->submodeID = 2;
-         else if (quality<=4)
-            st->submodeID = 3;
-         else if (quality<=6)
-            st->submodeID = 4;
-         else if (quality<=8)
-            st->submodeID = 5;
-         else if (quality<=9)
-            st->submodeID = 6;
-         else if (quality<=10)
-            st->submodeID = 7;
-         else
-         fprintf(stderr, "Unknown nb_ctl quality: %d\n", quality);*/
          if (quality < 0)
             quality = 0;
          if (quality > 10)
             quality = 10;
-         st->submodeID = ((SpeexNBMode*)(st->mode->mode))->quality_map[quality];
+         st->submodeSelect = st->submodeID = ((SpeexNBMode*)(st->mode->mode))->quality_map[quality];
       }
       break;
    case SPEEX_SET_COMPLEXITY:
@@ -1290,7 +1578,7 @@ void nb_encoder_ctl(void *state, int request, void *ptr)
       {
          int i=10, rate, target;
          target = (*(int*)ptr);
-         while (i>=1)
+         while (i>=0)
          {
             speex_encoder_ctl(st, SPEEX_SET_QUALITY, &i);
             speex_encoder_ctl(st, SPEEX_GET_BITRATE, &rate);
@@ -1312,6 +1600,19 @@ void nb_encoder_ctl(void *state, int request, void *ptr)
    case SPEEX_GET_SAMPLING_RATE:
       (*(int*)ptr)=st->sampling_rate;
       break;
+   case SPEEX_RESET_STATE:
+      {
+         int i;
+         st->bounded_pitch = 1;
+         st->first = 1;
+         for (i=0;i<st->lpcSize;i++)
+            st->lsp[i]=(M_PI*((float)(i+1)))/(st->lpcSize+1);
+         for (i=0;i<st->lpcSize;i++)
+            st->mem_sw[i]=st->mem_sw_whole[i]=st->mem_sp[i]=st->mem_exc[i]=0;
+         for (i=0;i<st->bufSize;i++)
+            st->excBuf[i]=st->swBuf[i]=st->inBuf[i]=st->exc2Buf[i]=0;
+      }
+      break;
    case SPEEX_GET_PI_GAIN:
       {
          int i;
@@ -1336,6 +1637,9 @@ void nb_encoder_ctl(void *state, int request, void *ptr)
             e[i]=st->innov[i];
       }
       break;
+   case SPEEX_GET_RELATIVE_QUALITY:
+      (*(float*)ptr)=st->relative_quality;
+      break;
    default:
       fprintf(stderr, "Unknown nb_ctl request: %d\n", request);
    }
@@ -1347,6 +1651,10 @@ void nb_decoder_ctl(void *state, int request, void *ptr)
    st=(DecState*)state;
    switch(request)
    {
+   case SPEEX_GET_LOW_MODE:
+   case SPEEX_GET_MODE:
+      (*(int*)ptr) = st->submodeID;
+      break;
    case SPEEX_SET_ENH:
       st->lpc_enh_enabled = *((int*)ptr);
       break;
@@ -1384,6 +1692,15 @@ void nb_decoder_ctl(void *state, int request, void *ptr)
          st->user_callback.callback_id=c->callback_id;
       }
       break;
+   case SPEEX_RESET_STATE:
+      {
+         int i;
+         for (i=0;i<2*st->lpcSize;i++)
+            st->mem_sp[i]=0;
+         for (i=0;i<st->bufSize;i++)
+            st->excBuf[i]=st->inBuf[i]=0;
+      }
+      break;
    case SPEEX_GET_PI_GAIN:
       {
          int i;
