@@ -27,6 +27,18 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: transports.cxx,v $
+ * Revision 1.128  2003/02/06 04:31:02  robertj
+ * Added more support for adding things to H323TransportAddressArrays
+ *
+ * Revision 1.127  2003/02/05 01:57:18  robertj
+ * Fixed STUN usage on gatekeeper discovery.
+ *
+ * Revision 1.126  2003/02/04 07:06:42  robertj
+ * Added STUN support.
+ *
+ * Revision 1.125  2003/01/23 02:36:32  robertj
+ * Increased (and made configurable) timeout for H.245 channel TCP connection.
+ *
  * Revision 1.124  2002/12/23 22:46:06  robertj
  * Changed gatekeeper discovery so an GRJ does not indicate "discovered".
  *
@@ -457,6 +469,8 @@
 #include "h323ep.h"
 #include "gkclient.h"
 
+#include <ptclib/pstun.h>
+
 
 class H225TransportThread : public PThread
 {
@@ -868,15 +882,38 @@ H323Transport * H323TransportAddress::CreateTransport(H323EndPoint & endpoint) c
 
 /////////////////////////////////////////////////////////////////////////////
 
+H323TransportAddressArray::H323TransportAddressArray(const H225_ArrayOf_TransportAddress & addresses)
+{
+  for (PINDEX i = 0; i < addresses.GetSize(); i++)
+    AppendAddress(H323TransportAddress(addresses[i]));
+}
+
+
+void H323TransportAddressArray::AppendString(const char * str)
+{
+  AppendAddress(H323TransportAddress(str));
+}
+
+
+void H323TransportAddressArray::AppendString(const PString & str)
+{
+  AppendAddress(H323TransportAddress(str));
+}
+
+
+void H323TransportAddressArray::AppendAddress(const H323TransportAddress & addr)
+{
+  if (!addr)
+    Append(new H323TransportAddress(addr));
+}
+
+
 void H323TransportAddressArray::AppendStringCollection(const PCollection & coll)
 {
   for (PINDEX i = 0; i < coll.GetSize(); i++) {
     PObject * obj = coll.GetAt(i);
-    if (obj != NULL && obj->IsDescendant(PString::Class())) {
-      H323TransportAddress addr = *(PString *)obj;
-      if (!addr)
-        Append(new H323TransportAddress(addr));
-    }
+    if (obj != NULL && obj->IsDescendant(PString::Class()))
+      AppendAddress(H323TransportAddress(*(PString *)obj));
   }
 }
 
@@ -1537,7 +1574,7 @@ BOOL H323TransportTCP::AcceptControlChannel(H323Connection & connection)
 
   PTCPSocket * h245Socket = new PTCPSocket;
 
-  h245listener->SetReadTimeout(30000); // Wait 30 seconds for remote H245 connect
+  h245listener->SetReadTimeout(endpoint.GetControlChannelStartTimeout());
   if (h245Socket->Accept(*h245listener))
     return Open(h245Socket);
 
@@ -1640,7 +1677,20 @@ BOOL H323TransportUDP::Connect()
   if (remoteAddress == 0 || remotePort == 0)
     return FALSE;
 
-  PUDPSocket * socket = (PUDPSocket *)GetReadChannel();
+  PUDPSocket * socket;
+
+  PSTUNClient * stun = endpoint.GetSTUN(remoteAddress);
+  if (stun != NULL) {
+    if (stun->CreateSocket(socket)) {
+      Open(socket);
+      socket->GetLocalAddress(localAddress, localPort);
+      PTRACE(4, "H323UDP\tSTUN created socket: " << localAddress << ':' << localPort);
+    }
+    else
+      PTRACE(4, "H323UDP\tSTUN could not create socket!");
+  }
+
+  socket = (PUDPSocket *)GetReadChannel();
   socket->SetSendAddress(remoteAddress, remotePort);
 
   return TRUE;
@@ -1765,6 +1815,8 @@ BOOL H323TransportUDP::DiscoverGatekeeper(H323Gatekeeper & gk,
   if (interfaces.IsEmpty())
     interfaces.Append(new PIPSocket::InterfaceEntry("", localAddress, PIPSocket::Address(0xffffffff), ""));
 
+  PSTUNClient * stun = endpoint.GetSTUN(remoteAddress);
+
   PSocketList sockets;
   PSocket::SelectList selection;
   H225_GatekeeperRequest & grq = request;
@@ -1789,14 +1841,21 @@ BOOL H323TransportUDP::DiscoverGatekeeper(H323Gatekeeper & gk,
     if (destAddr != MulticastRasAddress) {
 
       // Not explicitly multicast
-      socket = new PUDPSocket;
+      if (stun != NULL && stun->CreateSocket(socket)) {
+        socket->GetLocalAddress(localAddress, localPort);
+        PTRACE(4, "H323UDP\tSTUN created socket: " << localAddress << ':' << localPort);
+      }
+      else {
+        socket = new PUDPSocket;
+        if (!ListenUDP(*socket, endpoint, localAddress, originalLocalPort)) {
+          delete socket;
+          return FALSE;
+        }
+        localPort = socket->GetPort();
+      }
+
       sockets.Append(socket);
 
-      if (!ListenUDP(*socket, endpoint, localAddress, originalLocalPort))
-        return FALSE;
-
-      localPort = socket->GetPort();
-
 #ifndef __BEOS__
       if (destAddr == INADDR_BROADCAST) {
         if (!socket->SetOption(SO_BROADCAST, 1)) {
