@@ -41,6 +41,9 @@
 /************ Change log
  *
  * $Log: encoder-h261.cxx,v $
+ * Revision 1.18  2003/02/10 00:33:02  robertj
+ * Removed code for redundent class and constructor.
+ *
  * Revision 1.17  2002/05/17 01:47:33  dereks
  * backout the integer maths in the h261 codec.
  *
@@ -88,7 +91,7 @@
 
 
 //static const char rcsid[] =
-//    "@(#) $Header: /home/cvsroot/openh323/src/vic/encoder-h261.cxx,v 1.17 2002/05/17 01:47:33 dereks Exp $ (LBL)";
+//    "@(#) $Header: /home/cvsroot/openh323/src/vic/encoder-h261.cxx,v 1.18 2003/02/10 00:33:02 robertj Exp $ (LBL)";
 
 
 #include "encoder-h261.h"
@@ -645,8 +648,7 @@ int H261DCTEncoder::consume(const VideoFrame *vf)
 {
 	if (!SameSize(vf))
 		SetSize(vf->width, vf->height);
-	DCTFrame* df = (DCTFrame *)vf;
-	return(encode(df, df->crvec));
+	return(encode(vf, vf->crvec));
 }
 
 int H261PixelEncoder::consume(const VideoFrame *vf)
