@@ -54,6 +54,9 @@
 /************ Change log
  *
  * $Log: p64.cxx,v $
+ * Revision 1.11  2002/10/10 05:40:29  robertj
+ * VxWorks port, thanks Martijn Roest
+ *
  * Revision 1.10  2002/04/26 04:57:41  dereks
  * Add Walter Whitlocks fixes, based on Victor Ivashim's suggestions to
  * improve the quality with Netmeeting. Thanks guys!!!!
@@ -84,7 +87,7 @@
 
 #ifndef lint
 //static char rcsid[] =
-//    "@(#) $Header: /home/cvsroot/openh323/src/vic/p64.cxx,v 1.10 2002/04/26 04:57:41 dereks Exp $ (LBL)";
+//    "@(#) $Header: /home/cvsroot/openh323/src/vic/p64.cxx,v 1.11 2002/10/10 05:40:29 robertj Exp $ (LBL)";
 #endif
 
 #include <stdarg.h>
@@ -96,7 +99,6 @@
 #if defined(WIN32) || defined(_WIN32_WCE)
 #include <winsock.h>
 #else
-#include <sys/param.h>
 #include <sys/file.h>
 #endif
 #include <sys/stat.h>
